@@ -1,5 +1,5 @@
 /* Command processing for GNU Make.
-Copyright (C) 1988, 1989, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
+Copyright (C) 1988, 89, 91, 92, 93, 94, 95, 96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -18,12 +18,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
 #include "dep.h"
-#include "commands.h"
-#include "file.h"
+#include "filedef.h"
 #include "variable.h"
 #include "job.h"
+#include "commands.h"
 
-extern int remote_kill ();
+extern int remote_kill PARAMS ((int id, int sig));
 
 #ifndef	HAVE_UNISTD_H
 extern int getpid ();
@@ -154,8 +154,11 @@ set_file_variables (file)
 
 	bcopy (c, cp, len);
 	cp += len;
+#if VMS
+        *cp++ = ',';
+#else
 	*cp++ = ' ';
-
+#endif
 	if (! d->changed)
 	  qmark_len -= len + 1;	/* Don't space in $? for this one.  */
       }
@@ -193,13 +196,20 @@ set_file_variables (file)
 
 	bcopy (c, cp, len);
 	cp += len;
+#if VMS
+	*cp++ = ',';
+#else
 	*cp++ = ' ';
-
+#endif
 	if (d->changed)
 	  {
 	    bcopy (c, qp, len);
 	    qp += len;
+#if VMS
+	    *qp++ = ',';
+#else
 	    *qp++ = ' ';
+#endif
 	  }
       }
 
@@ -359,9 +369,16 @@ RETSIGTYPE
 fatal_error_signal (sig)
      int sig;
 {
-#ifdef	__MSDOS__
+#if defined(__MSDOS__) || defined(_AMIGA)
   remove_intermediates (1);
+#ifdef _AMIGA
+  if (sig == SIGINT)
+     fputs ("*** Break.\n", stderr);
+
+  exit (10);
+#else
   exit (1);
+#endif
 #else	/* Not MSDOS.  */
   handling_fatal_signal = 1;
 
@@ -383,7 +400,14 @@ fatal_error_signal (sig)
   /* If we got a signal that means the user
      wanted to kill make, remove pending targets.  */
 
-  if (sig == SIGTERM || sig == SIGINT || sig == SIGHUP || sig == SIGQUIT)
+  if (sig == SIGTERM || sig == SIGINT
+#ifdef SIGHUP
+    || sig == SIGHUP
+#endif
+#ifdef SIGQUIT
+    || sig == SIGQUIT
+#endif
+    )
     {
       register struct child *c;
 
@@ -410,10 +434,12 @@ fatal_error_signal (sig)
 
   remove_intermediates (1);
 
+#ifdef SIGQUIT
   if (sig == SIGQUIT)
     /* We don't want to send ourselves SIGQUIT, because it will
        cause a core dump.  Just exit instead.  */
-    exit (1);
+    exit (EXIT_FAILURE);
+#endif
 
   /* Signal the same code; this time it will really be fatal.  The signal
      will be unblocked when we return and arrive then to kill us.  */
@@ -451,7 +477,7 @@ delete_target (file, on_behalf_of)
     }
 #endif
 
-  if (safe_stat (file->name, &st) == 0
+  if (stat (file->name, &st) == 0
       && S_ISREG (st.st_mode)
       && (time_t) st.st_mtime != file->last_mtime)
     {
@@ -459,11 +485,12 @@ delete_target (file, on_behalf_of)
 	error ("*** [%s] Deleting file `%s'", on_behalf_of, file->name);
       else
 	error ("*** Deleting file `%s'", file->name);
-      if (unlink (file->name) < 0)
+      if (unlink (file->name) < 0
+	  && errno != ENOENT)	/* It disappeared; so what.  */
 	perror_with_name ("unlink: ", file->name);
     }
 }
-     
+
 
 /* Delete all non-precious targets of CHILD unless they were already deleted.
    Set the flag in CHILD to say they've been deleted.  */
