@@ -1,5 +1,5 @@
 /* Variable function expansion for GNU Make.
-Copyright (C) 1988, 89, 91, 92, 93, 94, 95 Free Software Foundation, Inc.
+Copyright (C) 1988, 89, 91, 92, 93, 94, 95, 96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -17,17 +17,26 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
+#include "filedef.h"
 #include "variable.h"
 #include "dep.h"
-#include "commands.h"
 #include "job.h"
+#include "commands.h"
 
 #ifdef __MSDOS__
 #include <process.h>
 #include <fcntl.h>
 #endif
+#ifdef _AMIGA
+#include "amiga.h"
+#endif
+#ifdef WIN32
+#include <windows.h>
+#include <io.h>
+#include "sub_proc.h"
+#endif
 
-static char *string_glob ();
+static char *string_glob PARAMS ((char *line));
 
 /* Store into VARIABLE_BUFFER at O the result of scanning TEXT and replacing
    each occurrence of SUBST with REPLACE. TEXT is null-terminated.  SLEN is
@@ -35,7 +44,7 @@ static char *string_glob ();
    nonzero, substitutions are done only on matches which are complete
    whitespace-delimited words.  If SUFFIX_ONLY is nonzero, substitutions are
    done only at the ends of whitespace-delimited words.  */
-   
+
 char *
 subst_expand (o, text, subst, replace, slen, rlen, by_word, suffix_only)
      char *o;
@@ -113,9 +122,9 @@ patsubst_expand (o, text, pattern, replace, pattern_percent, replace_percent)
      register char *pattern, *replace;
      register char *pattern_percent, *replace_percent;
 {
-  register int pattern_prepercent_len, pattern_postpercent_len;
-  register int replace_prepercent_len, replace_postpercent_len;
-  register char *t;
+  unsigned int pattern_prepercent_len, pattern_postpercent_len;
+  unsigned int replace_prepercent_len, replace_postpercent_len;
+  char *t;
   unsigned int len;
   int doneany = 0;
 
@@ -337,24 +346,38 @@ expand_function (o, function, text, end)
     default:
       abort ();
       break;
-      
+
+#ifndef VMS /* not supported for vms yet */
     case function_shell:
       {
-	char **argv, **envp;
+#ifdef WIN32
+        SECURITY_ATTRIBUTES saAttr;
+        HANDLE hIn;
+        HANDLE hErr;
+        HANDLE hChildOutRd;
+        HANDLE hChildOutWr;
+        HANDLE hProcess;
+#endif
+	char **argv;
 	char *error_prefix;
+#ifndef _AMIGA
+	char **envp;
 	int pipedes[2];
 	int pid;
+#endif
 
 	/* Expand the command line.  */
 	text = expand_argument (text, end);
 
 	/* Construct the argument list.  */
-	argv = construct_command_argv (text, (char *) NULL, (struct file *) 0);
+	argv = construct_command_argv (text,
+				       (char **) NULL, (struct file *) 0);
 	if (argv == 0)
 	  break;
 
+#ifndef _AMIGA
 	/* Using a target environment for `shell' loses in cases like:
-	   	export var = $(shell echo foobie) 
+	   	export var = $(shell echo foobie)
 	   because target_environment hits a loop trying to expand $(var)
 	   to put it in the environment.  This is even more confusing when
 	   var was not explicitly exported, but just appeared in the
@@ -365,6 +388,7 @@ expand_function (o, function, text, end)
 	/* Construct the environment.  */
 	envp = target_environment ((struct file *) 0);
 #endif
+#endif	/* Not Amiga.  */
 
 	/* For error messages.  */
 	if (reading_filename != 0)
@@ -376,7 +400,58 @@ expand_function (o, function, text, end)
 	else
 	  error_prefix = "";
 
-#ifndef	  __MSDOS__
+#if !defined(__MSDOS__) && !defined(_AMIGA) 
+# ifdef WIN32
+        saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
+        saAttr.bInheritHandle = TRUE;
+        saAttr.lpSecurityDescriptor = NULL;
+
+        if (DuplicateHandle(GetCurrentProcess(),
+                            GetStdHandle(STD_INPUT_HANDLE),
+                            GetCurrentProcess(),
+                            &hIn,
+                            0,
+                            TRUE,
+                            DUPLICATE_SAME_ACCESS) == FALSE) {
+          fatal("create_child_process: DuplicateHandle(In) failed (e=%d)\n",
+                GetLastError());
+        }
+        if (DuplicateHandle(GetCurrentProcess(),
+                            GetStdHandle(STD_ERROR_HANDLE),
+                            GetCurrentProcess(),
+                            &hErr,
+                            0,
+                            TRUE,
+                            DUPLICATE_SAME_ACCESS) == FALSE) {
+          fatal("create_child_process: DuplicateHandle(Err) failed (e=%d)\n",
+                GetLastError());
+        }
+
+        if (!CreatePipe(&hChildOutRd, &hChildOutWr, &saAttr, 0))
+          fatal("CreatePipe() failed (e=%d)\n", GetLastError());
+
+        hProcess = process_init_fd(hIn, hChildOutWr, hErr);
+
+        if (!hProcess)
+          fatal("expand_function: process_init_fd() failed\n");
+        else
+          process_register(hProcess);
+
+        /* make sure that CreateProcess() has Path it needs */
+        sync_Path_environment();
+
+        if (!process_begin(hProcess, argv, envp, argv[0], NULL))
+                pid = (int) hProcess;
+        else
+                fatal("expand_function: unable to launch process (e=%d)\n",
+                      process_last_err(hProcess));
+
+        /* set up to read data from child */
+        pipedes[0] = _open_osfhandle((long) hChildOutRd, O_RDONLY);
+
+        /* this will be closed almost right away */
+        pipedes[1] = _open_osfhandle((long) hChildOutWr, O_APPEND);
+# else /* WIN32 */
 	if (pipe (pipedes) < 0)
 	  {
 	    perror_with_name (error_prefix, "pipe");
@@ -389,6 +464,7 @@ expand_function (o, function, text, end)
 	else if (pid == 0)
 	  child_execute_job (0, pipedes[1], argv, envp);
 	else
+# endif /* WIN32 */
 	  {
 	    /* We are the parent.  */
 
@@ -481,7 +557,8 @@ expand_function (o, function, text, end)
 
 	    free (buffer);
 	  }
-#else	/* MSDOS.  */
+#else	/* MSDOS or Amiga */
+#ifndef _AMIGA
          {
 	   /* MS-DOS can't do fork, but it can do spawn.  However, this
 	      means that we don't have an opportunity to reopen stdout to
@@ -498,7 +575,7 @@ expand_function (o, function, text, end)
 	   unsigned int maxlen = 200;
 	   int cc;
 	   char *buffer;
- 
+
 	   strcpy (tmp_output, "shXXXXXX");
 	   mktemp (tmp_output);
 	   child_stdout = open (tmp_output,
@@ -509,9 +586,9 @@ expand_function (o, function, text, end)
 	   dup2 (save_stdout, 1);
 	   close (child_stdout);
 	   close (save_stdout);
- 
+
 	   child_stdout = open (tmp_output, O_RDONLY|O_TEXT, 0644);
- 
+
 	   buffer = xmalloc (maxlen);
 	   i = 0;
 	   do
@@ -521,15 +598,15 @@ expand_function (o, function, text, end)
 		   maxlen += 512;
 		   buffer = (char *) xrealloc (buffer, maxlen + 1);
 		 }
- 
+
 	       cc = read (child_stdout, &buffer[i], maxlen - i);
 	       if (cc > 0)
 		 i += cc;
 	     } while (cc > 0);
- 
+
 	   close (child_stdout);
 	   unlink (tmp_output);
- 
+
 	   if (i > 0)
 	     {
 	       if (buffer[i - 1] == '\n')
@@ -543,11 +620,88 @@ expand_function (o, function, text, end)
 	     }
 	   free (buffer);
 	 }
-#endif	/* Not MSDOS.  */
+#else /* Amiga */
+	 {
+	   /* Amiga can't fork nor spawn, but I can start a program with
+	      redirection of my choice. The rest is the same as above. */
+#include <dos/dos.h>
+#include <proto/dos.h>
+
+	   BPTR child_stdout;
+	   char tmp_output[FILENAME_MAX];
+	   unsigned int maxlen = 200;
+	   int cc;
+	   char * buffer, * ptr;
+	   char ** aptr;
+	   int len = 0;
+
+	   strcpy (tmp_output, "t:MakeshXXXXXXXX");
+	   mktemp (tmp_output);
+	   child_stdout = Open (tmp_output, MODE_NEWFILE);
+
+	   for (aptr=argv; *aptr; aptr++)
+	     {
+	       len += strlen (*aptr) + 1;
+	     }
+
+	   buffer = xmalloc (len + 1);
+	   ptr = buffer;
+
+	   for (aptr=argv; *aptr; aptr++)
+	     {
+	       strcpy (ptr, *aptr);
+	       ptr += strlen (ptr) + 1;
+	       *ptr ++ = ' ';
+	       *ptr = 0;
+	     }
+
+	   ptr[-1] = '\n';
+
+	   Execute (buffer, NULL, child_stdout);
+	   free (buffer);
+
+	   Close (child_stdout);
+
+	   child_stdout = Open (tmp_output, MODE_OLDFILE);
+
+	   buffer = xmalloc (maxlen);
+	   i = 0;
+	   do
+	     {
+	       if (i == maxlen)
+		 {
+		   maxlen += 512;
+		   buffer = (char *) xrealloc (buffer, maxlen + 1);
+		 }
+
+	       cc = Read (child_stdout, &buffer[i], maxlen - i);
+	       if (cc > 0)
+		 i += cc;
+	     } while (cc > 0);
+
+	   Close (child_stdout);
+	   DeleteFile (tmp_output);
+
+	   if (i > 0)
+	     {
+	       if (buffer[i - 1] == '\n')
+		 buffer[--i] = '\0';
+	       else
+		 buffer[i] = '\0';
+	       p = buffer;
+	       while ((p = index (p, '\n')) != 0)
+		 *p++ = ' ';
+	       o = variable_buffer_output (o, buffer, i);
+	     }
+	   free (buffer);
+	 }
+#endif	/* Not Amiga.  */
+#endif	/* MSDOS or Amiga.  */
 
 	free (text);
 	break;
       }
+#endif /* !VMS */
 
     case function_origin:
       /* Expand the argument.  */
@@ -590,7 +744,7 @@ expand_function (o, function, text, end)
 
       free (text);
       break;
-      
+
     case function_sort:
       /* Expand the argument.  */
       text = expand_argument (text, end);
@@ -610,7 +764,7 @@ expand_function (o, function, text, end)
 		nwords *= 2;
 		words = (char **) xrealloc ((char *) words,
 					    nwords * sizeof (char *));
-	      }	
+	      }
 	    words[wordi++] = savestring (p, len);
 	  }
 
@@ -640,7 +794,7 @@ expand_function (o, function, text, end)
 
       free (text);
       break;
-      
+
     case function_foreach:
       {
 	/* Get three comma-separated arguments but
@@ -733,7 +887,7 @@ expand_function (o, function, text, end)
 	if (p == end)
 	  BADARGS (function == function_filter ? "filter" : "filter-out");
 	p2 = expand_argument (text, p);
-	
+
 	text = expand_argument (p + 1, end);
 
 	/* Chop TEXT up into words and then run each pattern through.  */
@@ -792,7 +946,7 @@ expand_function (o, function, text, end)
 	free (text);
       }
       break;
-      
+
     case function_patsubst:
       /* Get three comma-separated arguments and expand each one.  */
       count = 0;
@@ -825,9 +979,9 @@ expand_function (o, function, text, end)
       text = expand_argument (text, p2);
       p3 = expand_argument (p2 + 1, p);
       p2 = expand_argument (p + 1, end);
-      
+
       o = patsubst_expand (o, p2, text, p3, (char *) 0, (char *) 0);
-      
+
       free (text);
       free (p3);
       free (p2);
@@ -850,7 +1004,7 @@ expand_function (o, function, text, end)
       text = expand_argument (text, p);
 
       p = expand_argument (p + 1, end);
-      
+
       {
 	/* Write each word of the first argument directly followed
 	   by the corresponding word of the second argument.
@@ -866,11 +1020,11 @@ expand_function (o, function, text, end)
 	    tp = find_next_token (&p2, &tlen);
 	    if (tp != 0)
 	      o = variable_buffer_output (o, tp, tlen);
-	    
+
 	    pp = find_next_token (&p3, &plen);
 	    if (pp != 0)
 	      o = variable_buffer_output (o, pp, plen);
-	    
+
 	    if (tp != 0 || pp != 0)
 	      {
 		o = variable_buffer_output (o, " ", 1);
@@ -882,11 +1036,11 @@ expand_function (o, function, text, end)
 	  /* Kill the last blank.  */
 	  --o;
       }
-      
+
       free (text);
       free (p);
       break;
-      
+
     case function_strip:
       /* Expand the argument.  */
       text = expand_argument (text, end);
@@ -901,19 +1055,23 @@ expand_function (o, function, text, end)
       if (doneany)
 	/* Kill the last space.  */
 	--o;
-      
+
       free (text);
       break;
-      
+
     case function_wildcard:
       text = expand_argument (text, end);
-      
+
+#ifdef _AMIGA
+      o = wildcard_expansion (text, o);
+#else
       p = string_glob (text);
       o = variable_buffer_output (o, p, strlen (p));
-      
+#endif
+
       free (text);
       break;
-      
+
     case function_subst:
       /* Get three comma-separated arguments and expand each one.  */
       count = 0;
@@ -946,14 +1104,14 @@ expand_function (o, function, text, end)
       text = expand_argument (text, p2);
       p3 = expand_argument (p2 + 1, p);
       p2 = expand_argument (p + 1, end);
-      
+
       o = subst_expand (o, p2, text, p3, strlen (text), strlen (p3), 0, 0);
-      
+
       free (text);
       free (p3);
       free (p2);
       break;
-      
+
     case function_firstword:
       /* Expand the argument.  */
       text = expand_argument (text, end);
@@ -963,10 +1121,10 @@ expand_function (o, function, text, end)
       p = find_next_token (&p2, &i);
       if (p != 0)
 	o = variable_buffer_output (o, p, i);
-      
+
       free (text);
       break;
-      
+
     case function_word:
       /* Get two comma-separated arguments and expand each one.  */
       count = 0;
@@ -1058,11 +1216,11 @@ index argument");
       i = strlen (text);
       if (sindex (p, 0, text, i) != 0)
 	o = variable_buffer_output (o, text, i);
-      
+
       free (p);
       free (text);
       break;
-      
+
     case function_addsuffix:
     case function_addprefix:
       /* Get two comma-separated arguments and expand each one.  */
@@ -1082,7 +1240,7 @@ index argument");
       i = strlen (text);
 
       p2 = expand_argument (p + 1, end);
-      
+
       p3 = p2;
       while ((p = find_next_token (&p3, &len)) != 0)
 	{
@@ -1097,11 +1255,11 @@ index argument");
       if (doneany)
 	/* Kill last space.  */
 	--o;
-      
+
       free (p2);
       free (text);
       break;
-      
+
     case function_dir:
     case function_basename:
       /* Expand the argument.  */
@@ -1111,7 +1269,11 @@ index argument");
       while ((p2 = find_next_token (&p3, &len)) != 0)
 	{
 	  p = p2 + len;
+#ifdef VMS
+	  while (p >= p2 && *p != (function == function_dir ? ']' : '.'))
+#else
 	  while (p >= p2 && *p != (function == function_dir ? '/' : '.'))
+#endif
 	    --p;
 	  if (p >= p2)
 	    {
@@ -1120,7 +1282,15 @@ index argument");
 	      o = variable_buffer_output (o, p2, p - p2);
 	    }
 	  else if (function == function_dir)
+#ifdef VMS
+            o = variable_buffer_output (o, "[]", 2);
+#else
+#ifndef _AMIGA
             o = variable_buffer_output (o, "./", 2);
+#else
+	    /* o = o */; /* Just a nop... */
+#endif /* AMIGA */
+#endif /* !VMS */
 	  else
 	    /* The entire name is the basename.  */
 	    o = variable_buffer_output (o, p2, len);
@@ -1131,10 +1301,10 @@ index argument");
       if (doneany)
 	/* Kill last space.  */
 	--o;
-      
+
       free (text);
       break;
-      
+
     case function_notdir:
     case function_suffix:
       /* Expand the argument.  */
@@ -1144,7 +1314,11 @@ index argument");
       while ((p2 = find_next_token (&p3, &len)) != 0)
 	{
 	  p = p2 + len;
+#ifdef VMS
+	  while (p >= p2 && *p != (function == function_notdir ? ']' : '.'))
+#else
 	  while (p >= p2 && *p != (function == function_notdir ? '/' : '.'))
+#endif
 	    --p;
 	  if (p >= p2)
 	    {
