@@ -1,5 +1,5 @@
 /* Argument parsing and main program of GNU Make.
-Copyright (C) 1988, 89, 90, 91, 94 Free Software Foundation, Inc.
+Copyright (C) 1988, 89, 90, 91, 94, 95, 96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -17,19 +17,37 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
-#include "commands.h"
 #include "dep.h"
-#include "file.h"
+#include "filedef.h"
 #include "variable.h"
 #include "job.h"
+#include "commands.h"
 #include "getopt.h"
+#include <assert.h>
+#ifdef _AMIGA
+#   include <dos/dos.h>
+#   include <proto/dos.h>
+#endif
+#ifdef WIN32
+#include <windows.h>
+#include "pathstuff.h"
+#endif
 
+#ifdef _AMIGA
+int __stack = 20000; /* Make sure we have 20K of stack space */
+#endif
 
-extern void print_variable_data_base ();
-extern void print_dir_data_base ();
-extern void print_rule_data_base ();
-extern void print_file_data_base ();
-extern void print_vpath_data_base ();
+extern void init_dir PARAMS ((void));
+extern void remote_setup PARAMS ((void));
+extern void remote_cleanup PARAMS ((void));
+extern RETSIGTYPE fatal_error_signal PARAMS ((int sig));
+extern RETSIGTYPE child_handler PARAMS ((int sig));
+
+extern void print_variable_data_base PARAMS ((void));
+extern void print_dir_data_base PARAMS ((void));
+extern void print_rule_data_base PARAMS ((void));
+extern void print_file_data_base PARAMS ((void));
+extern void print_vpath_data_base PARAMS ((void));
 
 #ifndef	HAVE_UNISTD_H
 extern int chdir ();
@@ -42,11 +60,12 @@ extern double atof ();
 #endif
 extern char *mktemp ();
 
-static void log_working_directory ();
-static void print_data_base (), print_version ();
-static void decode_switches (), decode_env_switches ();
-static void define_makeflags ();
-static char *quote_as_word ();
+static void print_data_base PARAMS((void));
+static void print_version PARAMS ((void));
+static void decode_switches PARAMS ((int argc, char **argv, int env));
+static void decode_env_switches PARAMS ((char *envar, unsigned int len));
+static void define_makeflags PARAMS ((int all, int makefile));
+static char *quote_as_word PARAMS ((char *out, char *in, int double_dollars));
 
 /* The structure that describes an accepted command switch.  */
 
@@ -110,6 +129,12 @@ int just_print_flag;
 
 int debug_flag = 0;
 
+#ifdef WIN32
+/* Suspend make in main for a short time to allow debugger to attach */
+
+int suspend_flag = 0;
+#endif
+
 /* Environment variables override makefile definitions.  */
 
 int env_overrides = 0;
@@ -169,9 +194,13 @@ static unsigned int inf_jobs = 0;
    Negative values mean unlimited, while zero means limit to
    zero load (which could be useful to start infinite jobs remotely
    but one at a time locally).  */
-
+#ifndef NO_FLOAT
 double max_load_average = -1.0;
 double default_load_average = -1.0;
+#else
+int max_load_average = -1;
+int default_load_average = -1;
+#endif
 
 /* List of directories given with -C switches.  */
 
@@ -211,6 +240,11 @@ static const struct command_switch switches[] =
     { 'd', flag, (char *) &debug_flag, 1, 1, 0, 0, 0,
 	"debug", 0,
 	"Print lots of debugging information" },
+#ifdef WIN32
+    { 'D', flag, (char *) &suspend_flag, 1, 1, 0, 0, 0,
+        "suspend-for-debug", 0,
+        "Suspend process to allow a debugger to attach" },
+#endif
     { 'e', flag, (char *) &env_overrides, 1, 1, 0, 0, 0,
 	"environment-overrides", 0,
 	"Environment variables override makefiles" },
@@ -234,10 +268,17 @@ static const struct command_switch switches[] =
 	0, (char *) &default_keep_going_flag,
 	"keep-going", 0,
 	"Keep going when some targets can't be made" },
+#ifndef NO_FLOAT
     { 'l', floating, (char *) &max_load_average, 1, 1, 0,
 	(char *) &default_load_average, (char *) &default_load_average,
 	"load-average", "N",
 	"Don't start multiple jobs unless load is below N" },
+#else
+    { 'l', positive_int, (char *) &max_load_average, 1, 1, 0,
+	(char *) &default_load_average, (char *) &default_load_average,
+	"load-average", "N",
+	"Don't start multiple jobs unless load is below N" },
+#endif
     { 'm', ignore, 0, 0, 0, 0, 0, 0,
 	0, 0,
 	"-b" },
@@ -366,6 +407,9 @@ static struct file *
 enter_command_line_file (name)
      char *name;
 {
+  if (name[0] == '\0')
+    fatal ("empty string invalid as file name");
+
   if (name[0] == '~')
     {
       char *expanded = tilde_expand (name);
@@ -383,7 +427,7 @@ enter_command_line_file (name)
 	/* Skip following slashes: ".//foo" is "foo", not "/foo".  */
 	++name;
     }
-  
+
   if (*name == '\0')
     {
       /* It was all slashes!  Move back to the dot and truncate
@@ -397,25 +441,41 @@ enter_command_line_file (name)
   return enter_file (savestring (name, strlen (name)));
 }
 
+/* Toggle -d on receipt of SIGUSR1.  */
+
+static RETSIGTYPE
+debug_signal_handler (sig)
+     int sig;
+{
+  debug_flag = ! debug_flag;
+}
+
+#ifndef _AMIGA
 int
 main (argc, argv, envp)
      int argc;
      char **argv;
      char **envp;
+#else
+int main (int argc, char ** argv)
+#endif
 {
-  extern void init_dir ();
-  extern RETSIGTYPE fatal_error_signal (), child_handler ();
   register struct file *f;
   register unsigned int i;
   char **p;
   struct dep *read_makefiles;
   PATH_VAR (current_directory);
+#ifdef WIN32
+  extern int no_default_sh_exe;
+  char *unix_path = NULL;
+  char *win32_path = NULL;
+#endif
 
   default_goal_file = 0;
   reading_filename = 0;
   reading_lineno_ptr = 0;
-  
-#ifndef	HAVE_SYS_SIGLIST
+
+#if !defined (HAVE_STRSIGNAL) && !defined (HAVE_SYS_SIGLIST)
   signame_init ();
 #endif
 
@@ -437,8 +497,12 @@ main (argc, argv, envp)
   else									      \
     ADD_SIG (sig);
 
+#ifdef SIGHUP
   FATAL_SIG (SIGHUP);
+#endif
+#ifdef SIGQUIT
   FATAL_SIG (SIGQUIT);
+#endif
   FATAL_SIG (SIGINT);
   FATAL_SIG (SIGTERM);
 
@@ -467,18 +531,19 @@ main (argc, argv, envp)
 #endif	/* setvbuf reversed.  */
 #endif	/* setlinebuf missing.  */
 
-  /* Initialize the directory hashing code.  */
-  init_dir ();
-
   /* Figure out where this program lives.  */
 
   if (argv[0] == 0)
     argv[0] = "";
   if (argv[0][0] == '\0')
     program = "make";
-  else 
+  else
     {
+#ifdef VMS
+      program = rindex (argv[0], ']');
+#else
       program = rindex (argv[0], '/');
+#endif
 #ifdef __MSDOS__
       if (program == 0)
 	program = rindex (argv[0], '\\');
@@ -496,7 +561,11 @@ main (argc, argv, envp)
 
   /* Figure out where we are.  */
 
+#ifdef WIN32
+  if (getcwd_fs (current_directory, GET_PATH_MAX) == 0)
+#else
   if (getcwd (current_directory, GET_PATH_MAX) == 0)
+#endif
     {
 #ifdef	HAVE_GETCWD
       perror_with_name ("getcwd: ", "");
@@ -514,11 +583,18 @@ main (argc, argv, envp)
      done before $(MAKE) is are figured out so its definitions will not be
      one from the environment.  */
 
+#ifndef _AMIGA
   for (i = 0; envp[i] != 0; ++i)
     {
       register char *ep = envp[i];
       while (*ep != '=')
 	++ep;
+#ifdef WIN32
+      if (!strncmp(ep, "PATH", 4))
+        unix_path = &ep[5];
+      if (!strncmp(ep, "Path", 4))
+        win32_path = &ep[5];
+#endif
       /* The result of pointer arithmetic is cast to unsigned int for
 	 machines where ptrdiff_t is a different size that doesn't widen
 	 the same.  */
@@ -531,6 +607,43 @@ main (argc, argv, envp)
 	   be exported, because it was originally in the environment.  */
 	->export = v_export;
     }
+#ifdef WIN32
+    /*
+     * PATH defaults to Path iff PATH not found and Path is found.
+     */
+    if (!unix_path && win32_path)
+      define_variable("PATH", 4, win32_path, o_env, 1)->export = v_export;
+#endif
+#else /* For Amiga, read the ENV: device, ignoring all dirs */
+    {
+	BPTR env, file, old;
+	char buffer[1024];
+	int len;
+	__aligned struct FileInfoBlock fib;
+
+	env = Lock ("ENV:", ACCESS_READ);
+	if (env)
+	{
+	    old = CurrentDir (DupLock(env));
+	    Examine (env, &fib);
+
+	    while (ExNext (env, &fib))
+	    {
+		if (fib.fib_DirEntryType < 0) /* File */
+		{
+		    /* Define an empty variable. It will be filled in
+			variable_lookup(). Makes startup quite a bit
+			faster. */
+			define_variable (fib.fib_FileName,
+			    strlen (fib.fib_FileName),
+			"", o_env, 1)->export = v_export;
+		}
+	    }
+	    UnLock (env);
+	    UnLock(CurrentDir(old));
+	}
+    }
+#endif
 
   /* Decode the switches.  */
 
@@ -542,6 +655,14 @@ main (argc, argv, envp)
   decode_env_switches ("MFLAGS", 6);
 #endif
   decode_switches (argc, argv, 0);
+#ifdef WIN32
+  if (suspend_flag) {
+        fprintf(stderr, "%s (pid = %d)\n", argv[0], GetCurrentProcessId());
+        fprintf(stderr, "%s is suspending for 30 seconds...", argv[0]);
+        Sleep(30 * 1000);
+        fprintf(stderr, "done sleep(30). Continuing.\n");
+  }
+#endif
 
   /* Print version information.  */
 
@@ -552,16 +673,28 @@ main (argc, argv, envp)
   if (print_version_flag)
     die (0);
 
-#ifndef __MSDOS__
+#if !defined(__MSDOS__) && !defined(VMS)
   /* Set the "MAKE_COMMAND" variable to the name we were invoked with.
      (If it is a relative pathname with a slash, prepend our directory name
      so the result will run the same program regardless of the current dir.
      If it is a name with no slash, we can only hope that PATH did not
      find it in the current directory.)  */
-
+#ifdef WIN32
+  /*
+   * Convert from backslashes to forward slashes for
+   * programs like sh which don't like them. Shouldn't
+   * matter if the path is one way or the other for
+   * CreateProcess().
+   */
+  if (strpbrk(argv[0], "/:\\") ||
+      strstr(argv[0], "..") ||
+      !strncmp(argv[0], "//", 2))
+    argv[0] = strdup(w32ify(argv[0],1));
+#else /* WIN32 */
   if (current_directory[0] != '\0'
       && argv[0] != 0 && argv[0][0] != '/' && index (argv[0], '/') != 0)
     argv[0] = concat (current_directory, "/", argv[0]);
+#endif /* WIN32 */
 #endif
 
   /* The extra indirection through $(MAKE_COMMAND) is done
@@ -589,7 +722,7 @@ main (argc, argv, envp)
 	}
 
       /* Now allocate a buffer big enough and fill it.  */
-      p = value = alloca (len);
+      p = value = (char *) alloca (len);
       for (cv = command_variables; cv != 0; cv = cv->next)
 	{
 	  v = cv->variable;
@@ -634,6 +767,79 @@ main (argc, argv, envp)
 	  free (dir);
       }
 
+#ifdef WIN32
+  /*
+   * THIS BLOCK OF CODE MUST COME AFTER chdir() CALL ABOVE IN ORDER
+   * TO NOT CONFUSE THE DEPENDENCY CHECKING CODE IN implicit.c.
+   *
+   * The functions in dir.c can incorrectly cache information for "."
+   * before we have changed directory and this can cause file
+   * lookups to fail because the current directory (.) was pointing
+   * at the wrong place when it was first evaluated.
+   */
+
+  /*
+   * On Windows/NT, we don't have the luxury of a /bin directory that
+   * is mapped globally to every drive mounted to the system. Since make could
+   * be invoked from any drive, and we don't want to propogate /bin/sh
+   * to every single drive. Allow ourselves a chance to search for
+   * a value for default shell here (if the default path does not exist).
+   *
+   * The value of default_shell is set here, but it could get reset after
+   * the Makefiles are read in. See logic below where SHELL is checked
+   * after the call to read_all_makefiles() completes.
+   *
+   * The reason SHELL is set here is so that macros can be safely evaluated
+   * as makefiles are read in (some macros require $SHELL).
+   */
+
+  {
+    extern char *default_shell;
+
+    if (!file_exists_p(default_shell)) {
+      char *p;
+      struct variable *v = lookup_variable ("Path", 4);
+
+      /*
+       * Try and make sure we have a full path to default_shell before
+       * we parse makefiles.
+       */
+      if (v && v->value) {
+        PATH_VAR(sh_path);
+        char *ep;
+
+        p  = v->value;
+        ep = strchr(p, PATH_SEPARATOR_CHAR);
+
+        while (ep && *ep) {
+          *ep = '\0';
+
+          if (dir_file_exists_p(p, default_shell)) {
+            sprintf(sh_path, "%s/%s", p, default_shell);
+            default_shell = strdup(w32ify(sh_path,0));
+            no_default_sh_exe = 0;
+            *ep = PATH_SEPARATOR_CHAR;
+
+            /* terminate loop */
+            p += strlen(p);
+          } else {
+            *ep = PATH_SEPARATOR_CHAR;
+             p = ++ep;
+          }
+
+          ep = strchr(p, PATH_SEPARATOR_CHAR);
+        }
+
+        /* be sure to check last element of Path */
+        if (p && *p && dir_file_exists_p(p, default_shell)) {
+            sprintf(sh_path, "%s/%s", p, default_shell);
+            default_shell = strdup(w32ify(sh_path,0));
+            no_default_sh_exe = 0;
+        }
+      }
+    }
+  }
+#endif /* WIN32 */
   /* Figure out the level of recursion.  */
   {
     struct variable *v = lookup_variable ("MAKELEVEL", 9);
@@ -662,7 +868,11 @@ main (argc, argv, envp)
     starting_directory = current_directory;
   else
     {
+#ifdef WIN32
+      if (getcwd_fs (current_directory, GET_PATH_MAX) == 0)
+#else
       if (getcwd (current_directory, GET_PATH_MAX) == 0)
+#endif
 	{
 #ifdef	HAVE_GETCWD
 	  perror_with_name ("getcwd: ", "");
@@ -675,11 +885,6 @@ main (argc, argv, envp)
 	starting_directory = current_directory;
     }
 
-  /* Tell the user where he is.  */
-
-  if (print_directory_flag)
-    log_working_directory (1);
-
   /* Read any stdin makefiles into temporary files.  */
 
   if (makefiles != 0)
@@ -691,11 +896,21 @@ main (argc, argv, envp)
 	    /* This makefile is standard input.  Since we may re-exec
 	       and thus re-read the makefiles, we read standard input
 	       into a temporary file and read from that.  */
-	    static char name[] = "/tmp/GmXXXXXX";
 	    FILE *outfile;
 
 	    /* Make a unique filename.  */
+#ifdef HAVE_MKTEMP
+
+#ifdef VMS
+	    static char name[] = "sys$scratch:GmXXXXXX";
+#else
+	    static char name[] = "/tmp/GmXXXXXX";
+#endif
 	    (void) mktemp (name);
+#else
+	    static char name[L_tmpnam];
+	    (void) tmpnam (name);
+#endif
 
 	    outfile = fopen (name, "w");
 	    if (outfile == 0)
@@ -703,7 +918,7 @@ main (argc, argv, envp)
 	    while (!feof (stdin))
 	      {
 		char buf[2048];
-		int n = fread (buf, 1, sizeof(buf), stdin);
+		unsigned int n = fread (buf, 1, sizeof(buf), stdin);
 		if (n > 0 && fwrite (buf, 1, n, outfile) != n)
 		  pfatal_with_name ("fwrite (temporary file)");
 	      }
@@ -744,6 +959,11 @@ main (argc, argv, envp)
   (void) signal (SIGCLD, child_handler);
 #endif
 
+  /* Let the user send us SIGUSR1 to toggle the -d flag during the run.  */
+#ifdef SIGUSR1
+  (void) signal (SIGUSR1, debug_signal_handler);
+#endif
+
   /* Define the initial list of suffixes for old-style rules.  */
 
   set_default_suffixes ();
@@ -765,6 +985,53 @@ main (argc, argv, envp)
 
   define_makeflags (0, 0);
 
+#ifdef WIN32
+  /*
+   * Now that makefiles are parsed, see if a Makefile gave a
+   * value for SHELL and use that for default_shell instead if
+   * that filename exists. This should speed up the
+   * construct_argv_internal() function by avoiding unnecessary
+   * recursion.
+   */
+  {
+    struct variable *v = lookup_variable("SHELL", 5);
+    extern char* default_shell;
+
+    /*
+     * to change value:
+     *
+     * SHELL must be found, SHELL must be set, value of SHELL
+     * must be different from current value, and the
+     * specified file must exist. Whew!
+     */
+    if (v != 0 && *v->value != '\0') {
+      char *fn = recursively_expand(v);
+
+      if (fn && strcmp(fn, default_shell) && file_exists_p(fn)) {
+        char *p;
+
+        default_shell = fn;
+
+        /* if Makefile says SHELL is sh.exe, believe it */
+        if (strstr(default_shell, "sh.exe"))
+               no_default_sh_exe = 0;
+
+        /*
+         * Convert from backslashes to forward slashes so
+         * create_command_line_argv_internal() is not confused.
+         */
+        for (p = strchr(default_shell, '\\'); p; p = strchr(default_shell, '\\'))
+          *p = '/';
+      }
+    }
+  }
+  if (no_default_sh_exe && job_slots != 1) {
+    error("Do not specify -j or --jobs if sh.exe is not available.");
+    error("Resetting make for single job mode.");
+    job_slots = 1;
+  }
+#endif /* WIN32 */
+
   /* Define the default variables.  */
   define_default_variables ();
 
@@ -835,6 +1102,9 @@ main (argc, argv, envp)
 	}
     }
 
+  /* Initialize the remote job module.  */
+  remote_setup ();
+
   if (read_makefiles != 0)
     {
       /* Update any makefiles if necessary.  */
@@ -897,21 +1167,24 @@ main (argc, argv, envp)
 		d = d->next;
 	      }
 	  }
-      }	
+      }
 
       /* Set up `MAKEFLAGS' specially while remaking makefiles.  */
       define_makeflags (1, 1);
 
       switch (update_goal_chain (read_makefiles, 1))
 	{
+	case 1:
 	default:
-	  abort ();
-	  
+#define BOGUS_UPDATE_STATUS 0
+	  assert (BOGUS_UPDATE_STATUS);
+	  break;
+
 	case -1:
 	  /* Did nothing.  */
 	  break;
-	  
-	case 1:
+
+	case 2:
 	  /* Failed to update.  Figure out if we care.  */
 	  {
 	    /* Nonzero if any makefile was successfully remade.  */
@@ -984,8 +1257,7 @@ main (argc, argv, envp)
 	  if (print_data_base_flag)
 	    print_data_base ();
 
-	  if (print_directory_flag)
-	    log_working_directory (0);
+	  log_working_directory (0);
 
 	  if (makefiles != 0)
 	    {
@@ -1022,6 +1294,7 @@ main (argc, argv, envp)
 		fatal ("Couldn't change back to original directory.");
 	    }
 
+#ifndef _AMIGA
 	  for (p = environ; *p != 0; ++p)
 	    if (!strncmp (*p, "MAKELEVEL=", 10))
 	      {
@@ -1034,6 +1307,20 @@ main (argc, argv, envp)
 		sprintf (*p, "MAKELEVEL=%u", makelevel);
 		break;
 	      }
+#else /* AMIGA */
+	  {
+	    char buffer[256];
+	    int len;
+
+	    len = GetVar ("MAKELEVEL", buffer, sizeof (buffer), GVF_GLOBAL_ONLY);
+
+	    if (len != -1)
+	    {
+	    sprintf (buffer, "%u", makelevel);
+	      SetVar ("MAKELEVEL", buffer, -1, GVF_GLOBAL_ONLY);
+	    }
+	  }
+#endif
 
 	  if (debug_flag)
 	    {
@@ -1047,7 +1334,12 @@ main (argc, argv, envp)
 	  fflush (stdout);
 	  fflush (stderr);
 
+#ifndef _AMIGA
 	  exec_command (argv, environ);
+#else
+	  exec_command (argv);
+	  exit (0);
+#endif
 	  /* NOTREACHED */
 	}
     }
@@ -1085,15 +1377,16 @@ main (argc, argv, envp)
 	    /* Nothing happened.  */
 	  case 0:
 	    /* Updated successfully.  */
-	    status = 0;
+	    status = EXIT_SUCCESS;
 	    break;
 	  case 2:
-	    /* Updating failed.  */
-	    status = 2;
+	    /* Updating failed.  POSIX.2 specifies exit status >1 for this;
+	       but in VMS, there is only success and failure.  */
+	    status = EXIT_FAILURE ? 2 : EXIT_FAILURE;
 	    break;
 	  case 1:
 	    /* We are under -q and would run some commands.  */
-	    status = 1;
+	    status = EXIT_FAILURE;
 	    break;
 	  default:
 	    abort ();
@@ -1179,6 +1472,51 @@ init_switches ()
   long_options[i].name = 0;
 }
 
+static void
+handle_non_switch_argument (arg, env)
+     char *arg;
+     int env;
+{
+  /* Non-option argument.  It might be a variable definition.  */
+  struct variable *v;
+  if (arg[0] == '-' && arg[1] == '\0')
+    /* Ignore plain `-' for compatibility.  */
+    return;
+  v = try_variable_definition ((char *) 0, 0, arg, o_command);
+  if (v != 0)
+    {
+      /* It is indeed a variable definition.  Record a pointer to
+	 the variable for later use in define_makeflags.  */
+      struct command_variable *cv
+	= (struct command_variable *) xmalloc (sizeof (*cv));
+      cv->variable = v;
+      cv->next = command_variables;
+      command_variables = cv;
+    }
+  else if (! env)
+    {
+      /* Not an option or variable definition; it must be a goal
+	 target!  Enter it as a file and add it to the dep chain of
+	 goals.  */
+      struct file *f = enter_command_line_file (arg);
+      f->cmd_target = 1;
+
+      if (goals == 0)
+	{
+	  goals = (struct dep *) xmalloc (sizeof (struct dep));
+	  lastgoal = goals;
+	}
+      else
+	{
+	  lastgoal->next
+	    = (struct dep *) xmalloc (sizeof (struct dep));
+	  lastgoal = lastgoal->next;
+	}
+      lastgoal->name = 0;
+      lastgoal->file = f;
+    }
+}
+
 /* Decode switches from ARGC and ARGV.
    They came from the environment if ENV is nonzero.  */
 
@@ -1204,64 +1542,16 @@ decode_switches (argc, argv, env)
   /* Reset getopt's state.  */
   optind = 0;
 
-  c = 0;
   while (optind < argc)
     {
+      /* Parse the next argument.  */
+      c = getopt_long (argc, argv, options, long_options, (int *) 0);
       if (c == EOF)
-	{
-	  /* There are no more options according to getting getopt, but
-	     there are some arguments left.  Since we have asked for
-	     non-option arguments to be returned in order, I think this
-	     only happens when there is a "--" argument to prevent later
-	     argument from being options.  Since getopt has finished its
-	     job, just update its state variables for the next argument and
-	     set C as if it had returned 1, indicating a non-option
-	     argument.  */
-	  optarg = argv[optind++];
-	  c = 1;
-	}
-      else
-	/* Parse the next argument.  */
-	c = getopt_long (argc, argv, options, long_options, (int *) 0);
-
-      if (c == 1)
-	{
-	  /* Non-option argument.  It might be a variable definition.  */
-	  struct variable *v;
-	  v = try_variable_definition ((char *) 0, 0, optarg, o_command);
-	  if (v != 0)
-	    {
-	      /* It is indeed a variable definition.  Record a pointer to
-		 the variable for later use in define_makeflags.  */
-	      struct command_variable *cv
-		= (struct command_variable *) xmalloc (sizeof (*cv));
-	      cv->variable = v;
-	      cv->next = command_variables;
-	      command_variables = cv;
-	    }
-	  else if (! env)
-	    {
-	      /* Not an option or variable definition; it must be a goal
-		 target!  Enter it as a file and add it to the dep chain of
-		 goals.  */
-	      struct file *f = enter_command_line_file (optarg);
-	      f->cmd_target = 1;
-	  
-	      if (goals == 0)
-		{
-		  goals = (struct dep *) xmalloc (sizeof (struct dep));
-		  lastgoal = goals;
-		}
-	      else
-		{
-		  lastgoal->next
-		    = (struct dep *) xmalloc (sizeof (struct dep));
-		  lastgoal = lastgoal->next;
-		}
-	      lastgoal->name = 0;
-	      lastgoal->file = f;
-	    }
-	}
+	/* End of arguments, or "--" marker seen.  */
+	break;
+      else if (c == 1)
+	/* An argument not starting with a dash.  */
+	handle_non_switch_argument (optarg, env);
       else if (c == '?')
 	/* Bad option.  We will print a usage message and die later.
 	   But continue to parse the other options so the user can
@@ -1346,34 +1636,47 @@ positive integral argument",
 		      = *(unsigned int *) cs->noarg_value;
 		  break;
 
+#ifndef NO_FLOAT
 		case floating:
 		  if (optarg == 0 && optind < argc
 		      && (isdigit (argv[optind][0]) || argv[optind][0] == '.'))
 		    optarg = argv[optind++];
 
 		  if (doit)
-		    *(double *) cs->value_ptr		    
+		    *(double *) cs->value_ptr
 		      = (optarg != 0 ? atof (optarg)
 			 : *(double *) cs->noarg_value);
 
 		  break;
+#endif
 		}
-	    
+
 	      /* We've found the switch.  Stop looking.  */
 	      break;
 	    }
     }
 
+  /* There are no more options according to getting getopt, but there may
+     be some arguments left.  Since we have asked for non-option arguments
+     to be returned in order, this only happens when there is a "--"
+     argument to prevent later arguments from being options.  */
+  while (optind < argc)
+    handle_non_switch_argument (argv[optind++], env);
+
+
   if (!env && (bad || print_usage_flag))
     {
       /* Print a nice usage message.  */
+      FILE *usageto;
 
       if (print_version_flag)
 	print_version ();
 
-      fprintf (stderr, "Usage: %s [options] [target] ...\n", program);
+      usageto = bad ? stderr : stdout;
 
-      fputs ("Options:\n", stderr);
+      fprintf (usageto, "Usage: %s [options] [target] ...\n", program);
+
+      fputs ("Options:\n", usageto);
       for (cs = switches; cs->c != '\0'; ++cs)
 	{
 	  char buf[1024], shortarg[50], longarg[50], *p;
@@ -1444,11 +1747,11 @@ positive integral argument",
 	       line with the description, leaving at least two spaces.
 	       Print it on its own line instead.  */
 	    {
-	      fprintf (stderr, "%s\n", buf);
+	      fprintf (usageto, "%s\n", buf);
 	      buf[0] = '\0';
 	    }
 
-	  fprintf (stderr, "%*s%s.\n",
+	  fprintf (usageto, "%*s%s.\n",
 		   - DESCRIPTION_COLUMN,
 		   buf, cs->description);
 	}
@@ -1518,34 +1821,12 @@ decode_env_switches (envar, len)
   *p = '\0';
   argv[++argc] = 0;
 
-  if (argc == 2 && argv[1][0] != '-')
-    {
-      /* There is just one word in the value, and it is not a switch.
-	 Either this is the single-word form and we should prepend a dash
-	 before calling decode_switches, or this is the multi-word form and
-	 there is no dash because it is a variable definition.  */
-      struct variable *v;
-      v = try_variable_definition ((char *) 0, 0, argv[1], o_command);
-      if (v != 0)
-	{
-	  /* It was indeed a variable definition, and now it has been
-	     processed.  There is nothing for decode_switches to do.
-	     Record a pointer to the variable for later use in
-	     define_makeflags.  */
-	  struct command_variable *cv
-	    = (struct command_variable *) xmalloc (sizeof (*cv));
-	  cv->variable = v;
-	  cv->next = command_variables;
-	  command_variables = cv;
-	  return;
-	}
-
-      /* It wasn't a variable definition, so it's some switches without a
-	 leading dash.  Add one and pass it along to decode_switches.  We
-	 need permanent storage for this in case decode_switches saves
-	 pointers into the value.  */
-      argv[1] = concat ("-", argv[1], "");
-    }
+  if (argv[1][0] != '-' && index (argv[1], '=') == 0)
+    /* The first word doesn't start with a dash and isn't a variable
+       definition.  Add a dash and pass it along to decode_switches.  We
+       need permanent storage for this in case decode_switches saves
+       pointers into the value.  */
+    argv[1] = concat ("-", argv[1], "");
 
   /* Parse those words.  */
   decode_switches (argc, argv, 1);
@@ -1565,7 +1846,11 @@ quote_as_word (out, in, double_dollars)
 {
   while (*in != '\0')
     {
+#ifdef VMS
+      if (index ("^;'\"*?$<>(){}|&~`\\ \t\r\n\f\v", *in) != 0)
+#else
       if (index ("^;'\"*?[]$<>(){}|&~`\\ \t\r\n\f\v", *in) != 0)
+#endif
 	*out++ = '\\';
       if (double_dollars && *in == '$')
 	*out++ = '$';
@@ -1661,6 +1946,7 @@ define_makeflags (all, makefile)
 	    }
 	  break;
 
+#ifndef NO_FLOAT
 	case floating:
 	  if (all)
 	    {
@@ -1680,6 +1966,7 @@ define_makeflags (all, makefile)
 		}
 	    }
 	  break;
+#endif
 
 	case string:
 	  if (all)
@@ -1747,6 +2034,25 @@ define_makeflags (all, makefile)
       flags = flags->next;
     }
 
+  /* Define MFLAGS before appending variable definitions.  */
+
+  if (p == &flagstring[1])
+    /* No flags.  */
+    flagstring[0] = '\0';
+  else if (p[-1] == '-')
+    {
+      /* Kill the final space and dash.  */
+      p -= 2;
+      *p = '\0';
+    }
+  else
+    /* Terminate the string.  */
+    *p = '\0';
+
+  /* Since MFLAGS is not parsed for flags, there is no reason to
+     override any makefile redefinition.  */
+  (void) define_variable ("MFLAGS", 6, flagstring, o_env, 1);
+
   if (all && command_variables != 0)
     {
       /* Now write a reference to $(MAKEOVERRIDES), which contains all the
@@ -1782,7 +2088,10 @@ define_makeflags (all, makefile)
 	}
     }
   else if (p == &flagstring[1])
-    --p;
+    {
+      words = 0;
+      --p;
+    }
   else if (p[-1] == '-')
     /* Kill the final space and dash.  */
     p -= 2;
@@ -1790,10 +2099,10 @@ define_makeflags (all, makefile)
   *p = '\0';
 
   v = define_variable ("MAKEFLAGS", 9,
-		       /* If there is just a single word of switches,
-			  omit the leading dash unless it is a single
-			  long option with two leading dashes.  */
-		       &flagstring[(words == 1 && command_variables == 0
+		       /* If there are switches, omit the leading dash
+			  unless it is a single long option with two
+			  leading dashes.  */
+		       &flagstring[(flagstring[0] == '-'
 				    && flagstring[1] != '-')
 				   ? 1 : 0],
 		       /* This used to use o_env, but that lost when a
@@ -1808,9 +2117,6 @@ define_makeflags (all, makefile)
        We should not do this again on the second call, because that is
        after reading makefiles which might have done `unexport MAKEFLAGS'. */
     v->export = v_export;
-  /* Since MFLAGS is not parsed for flags, there is no reason to
-     override any makefile redefinition.  */
-  (void) define_variable ("MFLAGS", 6, flagstring, o_env, 1);
 }
 
 /* Print version information.  */
@@ -1831,10 +2137,13 @@ print_version ()
     printf ("-%s", remote_description);
 
   printf (", by Richard Stallman and Roland McGrath.\n\
-%sCopyright (C) 1988, 89, 90, 91, 92, 93, 94 Free Software Foundation, Inc.\n\
+%sCopyright (C) 1988, 89, 90, 91, 92, 93, 94, 95, 96\n\
+%s\tFree Software Foundation, Inc.\n\
 %sThis is free software; see the source for copying conditions.\n\
 %sThere is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
-%sPARTICULAR PURPOSE.\n\n", precede, precede, precede, precede);
+%sPARTICULAR PURPOSE.\n\n\
+%sReport bugs to <bug-gnu-utils@prep.ai.mit.edu>.\n\n",
+	  precede, precede, precede, precede, precede, precede);
 
   printed_version = 1;
 
@@ -1848,7 +2157,6 @@ print_version ()
 static void
 print_data_base ()
 {
-  extern char *ctime ();
   time_t when;
 
   when = time ((time_t *) 0);
@@ -1892,14 +2200,16 @@ die (status)
       for (err = status != 0; job_slots_used > 0; err = 0)
 	reap_children (1, err);
 
+      /* Let the remote job module clean up its state.  */
+      remote_cleanup ();
+
       /* Remove the intermediate files.  */
       remove_intermediates (0);
 
       if (print_data_base_flag)
 	print_data_base ();
 
-      if (print_directory_flag)
-	log_working_directory (0);
+      log_working_directory (0);
     }
 
   exit (status);
@@ -1908,20 +2218,21 @@ die (status)
 /* Write a message indicating that we've just entered or
    left (according to ENTERING) the current directory.  */
 
-static void
+void
 log_working_directory (entering)
      int entering;
 {
   static int entered = 0;
   char *message = entering ? "Entering" : "Leaving";
 
-  if (entering)
-    entered = 1;
-  else if (!entered)
-    /* Don't print the leaving message if we
-       haven't printed the entering message.  */
+  /* Print nothing without the flag.  Don't print the entering message
+     again if we already have.  Don't print the leaving message if we
+     haven't printed the entering message.  */
+  if (! print_directory_flag || entering == entered)
     return;
 
+  entered = entering;
+
   if (print_data_base_flag)
     fputs ("# ", stdout);
 
