@@ -50,9 +50,8 @@ collapse_continuations (line)
     return;
 
   out = in;
-  if (out > line)
-    while (out[-1] == '\\')
-      --out;
+  while (out > line && out[-1] == '\\')
+    --out;
 
   while (*in != '\0')
     {
@@ -174,15 +173,25 @@ concat (s1, s2, s3)
 /* Print a message on stdout.  */
 
 void
-message (s1, s2, s3, s4, s5, s6)
+message (prefix, s1, s2, s3, s4, s5, s6)
+     int prefix;
      char *s1, *s2, *s3, *s4, *s5, *s6;
 {
-  if (makelevel == 0)
-    printf ("%s: ", program);
-  else
-    printf ("%s[%u]: ", program, makelevel);
-  printf (s1, s2, s3, s4, s5, s6);
-  putchar ('\n');
+  log_working_directory (1);
+
+  if (s1 != 0)
+    {
+      if (prefix)
+	{
+	  if (makelevel == 0)
+	    printf ("%s: ", program);
+	  else
+	    printf ("%s[%u]: ", program, makelevel);
+	}
+      printf (s1, s2, s3, s4, s5, s6);
+      putchar ('\n');
+    }
+
   fflush (stdout);
 }
 
@@ -193,6 +202,8 @@ void
 fatal (s1, s2, s3, s4, s5, s6)
      char *s1, *s2, *s3, *s4, *s5, *s6;
 {
+  log_working_directory (1);
+
   if (makelevel == 0)
     fprintf (stderr, "%s: *** ", program);
   else
@@ -211,6 +222,8 @@ void
 error (s1, s2, s3, s4, s5, s6)
      char *s1, *s2, *s3, *s4, *s5, *s6;
 {
+  log_working_directory (1);
+
   if (makelevel == 0)
     fprintf (stderr, "%s: ", program);
   else
@@ -226,6 +239,8 @@ makefile_error (file, lineno, s1, s2, s3, s4, s5, s6)
      unsigned int lineno;
      char *s1, *s2, *s3, *s4, *s5, *s6;
 {
+  log_working_directory (1);
+
   fprintf (stderr, "%s:%u: ", file, lineno);
   fprintf (stderr, s1, s2, s3, s4, s5, s6);
   putc ('\n', stderr);
@@ -238,6 +253,8 @@ makefile_fatal (file, lineno, s1, s2, s3, s4, s5, s6)
      unsigned int lineno;
      char *s1, *s2, *s3, *s4, *s5, *s6;
 {
+  log_working_directory (1);
+
   fprintf (stderr, "%s:%u: *** ", file, lineno);
   fprintf (stderr, s1, s2, s3, s4, s5, s6);
   fputs (".  Stop.\n", stderr);
@@ -254,7 +271,9 @@ strerror (errnum)
      int errnum;
 {
   extern int errno, sys_nerr;
+#ifndef __DECC
   extern char *sys_errlist[];
+#endif
   static char buf[] = "Unknown error 12345678901234567890";
 
   if (errno < sys_nerr)
@@ -378,6 +397,37 @@ end_of_token (s)
   return s;
 }
 
+#ifdef WIN32
+/*
+ * Same as end_of_token, but take into account a stop character
+ */
+char *
+end_of_token_w32 (s, stopchar)
+     char *s;
+     char stopchar;
+{
+  register char *p = s;
+  register int backslash = 0;
+
+  while (*p != '\0' && *p != stopchar && (backslash || !isblank (*p)))
+    {
+      if (*p++ == '\\')
+        {
+          backslash = !backslash;
+          while (*p == '\\')
+            {
+              backslash = !backslash;
+              ++p;
+            }
+        }
+      else
+        backslash = 0;
+    }
+
+  return p;
+}
+#endif
+
 /* Return the address of the first nonwhitespace or null in the string S.  */
 
 char *
@@ -523,6 +573,7 @@ log_access (flavor)
 static void
 init_access ()
 {
+#ifndef VMS
   user_uid = getuid ();
   user_gid = getgid ();
 
@@ -536,6 +587,7 @@ init_access ()
   log_access ("Initialized");
 
   current_access = make;
+#endif
 }
 
 #endif	/* GETLOADAVG_PRIVILEGED */
@@ -716,23 +768,3 @@ get_path_max ()
   return value;
 }
 #endif
-
-/* On some systems, stat can return EINTR.  */
-
-int
-safe_stat (name, buf)
-     char *name;
-     struct stat *buf;
-{
-  int ret;
-
-#ifdef EINTR
-  do
-#endif
-    ret = stat (name, buf);
-#ifdef EINTR
-  while (ret < 0 && errno == EINTR);
-#endif
-
-  return  ret;
-}
