@@ -1,5 +1,5 @@
 /* Reading and parsing of makefiles for GNU Make.
-Copyright (C) 1988, 89, 90, 91, 92, 93, 94, 1995 Free Software Foundation, Inc.
+Copyright (C) 1988,89,90,91,92,93,94,95,96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -17,9 +17,10 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
-#include "commands.h"
 #include "dep.h"
-#include "file.h"
+#include "filedef.h"
+#include "job.h"
+#include "commands.h"
 #include "variable.h"
 
 /* This is POSIX.2, but most systems using -DPOSIX probably don't have it.  */
@@ -29,16 +30,15 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "glob/glob.h"
 #endif
 
+#ifndef WIN32
+#ifndef _AMIGA
+#ifndef VMS
 #include <pwd.h>
-struct passwd *getpwnam ();
-
-
-static int read_makefile ();
-static unsigned int readline (), do_define ();
-static int conditional_line ();
-static void record_files ();
-static char *find_semicolon ();
-
+#else
+struct passwd *getpwnam PARAMS ((char *name));
+#endif
+#endif
+#endif /* !WIN32 */
 
 /* A `struct linebuffer' is a structure which holds a line of text.
    `readline' reads a line from a stream into a linebuffer
@@ -75,16 +75,25 @@ struct conditionals
 
 static struct conditionals toplevel_conditionals;
 static struct conditionals *conditionals = &toplevel_conditionals;
-  
+
 
 /* Default directories to search for include files in  */
 
 static char *default_include_directories[] =
   {
+#if defined(WIN32) && !defined(INCLUDEDIR)
+/*
+ * This completly up to the user when they install MSVC or other packages.
+ * This is defined as a placeholder.
+ */
+#define INCLUDEDIR "."
+#endif
     INCLUDEDIR,
+#ifndef _AMIGA
     "/usr/gnu/include",
     "/usr/local/include",
     "/usr/include",
+#endif
     0
   };
 
@@ -105,6 +114,17 @@ unsigned int *reading_lineno_ptr;
 /* The chain of makefiles read by read_makefile.  */
 
 static struct dep *read_makefiles = 0;
+
+static int read_makefile PARAMS ((char *filename, int flags));
+static unsigned int readline PARAMS ((struct linebuffer *linebuffer, FILE *stream,
+			char *filename, unsigned int lineno));
+static unsigned int do_define PARAMS ((char *name, unsigned int namelen, enum variable_origin origin,
+			unsigned int lineno, FILE *infile, char *filename));
+static int conditional_line PARAMS ((char *line, char *filename, unsigned int lineno));
+static void record_files PARAMS ((struct nameseq *filenames, char *pattern, char *pattern_percent,
+			struct dep *deps, unsigned int commands_started, char *commands,
+			unsigned int commands_idx, int two_colon, char *filename,
+			unsigned int lineno, int set_default));
 
 /* Read in all the makefiles and return the chain of their names.  */
 
@@ -139,6 +159,7 @@ read_all_makefiles (makefiles)
     /* Set NAME to the start of next token and LENGTH to its length.
        MAKEFILES is updated for finding remaining tokens.  */
     p = value;
+
     while ((name = find_next_token (&p, &length)) != 0)
       {
 	if (*p != '\0')
@@ -177,7 +198,16 @@ read_all_makefiles (makefiles)
   if (num_makefiles == 0)
     {
       static char *default_makefiles[] =
+#ifdef VMS
+	/* all lower case since readdir() (the vms version) 'lowercasifies' */
+	{ "makefile.vms", "gnumakefile", "makefile", 0 };
+#else
+#ifdef _AMIGA
+	{ "GNUmakefile", "Makefile", "SMakefile", 0 };
+#else /* !Amiga && !VMS */
 	{ "GNUmakefile", "makefile", "Makefile", 0 };
+#endif /* AMIGA */
+#endif /* VMS */
       register char **p = default_makefiles;
       while (*p != 0 && !file_exists_p (*p))
 	++p;
@@ -192,6 +222,9 @@ read_all_makefiles (makefiles)
 	  /* No default makefile was found.  Add the default makefiles to the
 	     `read_makefiles' chain so they will be updated if possible.  */
 	  struct dep *tail = read_makefiles;
+	  /* Add them to the tail, after any MAKEFILES variable makefiles.  */
+	  while (tail != 0 && tail->next != 0)
+	    tail = tail->next;
 	  for (p = default_makefiles; *p != 0; ++p)
 	    {
 	      struct dep *d = (struct dep *) xmalloc (sizeof (struct dep));
@@ -250,6 +283,9 @@ read_makefile (filename, flags)
   char *pattern = 0, *pattern_percent;
 
   int makefile_errno;
+#ifdef WIN32
+  int check_again;
+#endif
 
 #define record_waiting_files()						      \
   do									      \
@@ -266,6 +302,7 @@ read_makefile (filename, flags)
 
 #ifdef	lint	/* Suppress `used before set' messages.  */
   two_colon = 0;
+  pattern_percent = 0;
 #endif
 
   if (debug_flag)
@@ -300,7 +337,6 @@ read_makefile (filename, flags)
   /* If the makefile wasn't found and it's either a makefile from
      the `MAKEFILES' variable or an included makefile,
      search the included makefile search path for this makefile.  */
-
   if (infile == 0 && (flags & RM_INCLUDED) && *filename != '/')
     {
       register unsigned int i;
@@ -542,15 +578,16 @@ read_makefile (filename, flags)
 	      v->export = v_noexport;
 	    }
 	}
-      else if (word1eq ("include", 7) || word1eq ("-include", 8))
+      else if (word1eq ("include", 7) || word1eq ("-include", 8)
+	       || word1eq ("sinclude", 8))
 	{
 	  /* We have found an `include' line specifying a nested
 	     makefile to be read at this point.  */
 	  struct conditionals *save, new_conditionals;
 	  struct nameseq *files;
-	  /* "-include" (vs "include") says no
-	     error if the file does not exist.  */
-	  int noerror = p[0] == '-';
+	  /* "-include" (vs "include") says no error if the file does not
+	     exist.  "sinclude" is an alias for this from SGI.  */
+	  int noerror = p[0] != 'i';
 
 	  p = allocated_variable_expand (next_token (p + (noerror ? 9 : 8)));
 	  if (*p == '\0')
@@ -584,7 +621,7 @@ read_makefile (filename, flags)
 	    {
 	      struct nameseq *next = files->next;
 	      char *name = files->name;
-	      free (files);
+	      free ((char *)files);
 	      files = next;
 
 	      if (! read_makefile (name, (RM_INCLUDED | RM_NO_TILDE
@@ -654,9 +691,16 @@ read_makefile (filename, flags)
 
 	  record_waiting_files ();
 
-	  /* Look for a semicolon in the unexpanded line.  */
-	  cmdleft = find_semicolon (lb.buffer);
-	  if (cmdleft != 0)
+	  /* Search the line for an unquoted ; that is not after an
+             unquoted #.  */
+	  cmdleft = find_char_unquote (lb.buffer, ";#", 0);
+	  if (cmdleft != 0 && *cmdleft == '#')
+	    {
+	      /* We found a comment before a semicolon.  */
+	      *cmdleft = '\0';
+	      cmdleft = 0;
+	    }
+	  else if (cmdleft != 0)
 	    /* Found one.  Cut the line short there before expanding it.  */
 	    *cmdleft = '\0';
 
@@ -668,15 +712,12 @@ read_makefile (filename, flags)
 
 	  if (cmdleft == 0)
 	    /* Look for a semicolon in the expanded line.  */
-	    cmdleft = find_semicolon (p);
+	    cmdleft = find_char_unquote (p, ";", 0);
 
 	  if (cmdleft != 0)
 	    /* Cut the line short at the semicolon.  */
 	    *cmdleft = '\0';
 
-	  /* Remove comments from the line.  */
-	  remove_comments (p);
-
 	  p2 = next_token (p);
 	  if (*p2 == '\0')
 	    {
@@ -727,6 +768,32 @@ read_makefile (filename, flags)
 	  /* For MS-DOS, skip a "C:\...".  */
 	  if (p != 0 && p[1] == '\\' && isalpha (p[-1]))
 	    p = 0;
+#endif
+#ifdef _AMIGA
+	  /* Here, the situation is quite complicated. Let's have a look
+	    at a couple of targets:
+
+		install: dev:make
+
+		dev:make: make
+
+		dev:make:: xyz
+
+	    The rule is that it's only a target, if there are TWO :'s
+	    OR a space around the :.
+	  */
+	  if (p && !(isspace(p[1]) || !p[1] || isspace(p[-1])))
+	    p = 0;
+#endif
+#ifdef WIN32
+          do {
+            check_again = 0;
+            /* For WIN32, skip a "C:\..." or a "C:/..." */
+            if (p != 0 && (p[1] == '\\' || p[1] == '/') && isalpha (p[-1])) {
+              p = index(p + 1, ':');
+              check_again = 1;
+            }
+          } while (check_again);
 #endif
 	  if (p != 0)
 	    {
@@ -1024,7 +1091,16 @@ conditional_line (line, filename, lineno)
       if (*line == '\0')
 	return -1;
 
-      *line++ = '\0';
+      if (termin == ',')
+	{
+	  /* Strip blanks after the first string.  */
+	  char *p = line++;
+	  while (isblank (p[-1]))
+	    --p;
+	  *p = '\0';
+	}
+      else
+	*line++ = '\0';
 
       s2 = variable_expand (s1);
       /* We must allocate a new copy of the expanded string because
@@ -1248,7 +1324,7 @@ record_files (filenames, pattern, pattern_percent, deps, commands_started,
 		d->name = savestring (buffer, o - buffer);
 	      }
 	  }
-      
+
       if (!two_colon)
 	{
 	  /* Single-colon.  Combine these dependencies
@@ -1297,7 +1373,7 @@ record_files (filenames, pattern, pattern_percent, deps, commands_started,
 		{
 		  struct dep *nextd = d->next;
  		  free (d->name);
- 		  free (d);
+ 		  free ((char *)d);
 		  d = nextd;
 		}
 	      f->deps = 0;
@@ -1442,7 +1518,7 @@ find_char_unquote (string, stopchars, blank)
      char *stopchars;
      int blank;
 {
-  unsigned int string_len = strlen (string);
+  unsigned int string_len = 0;
   register char *p = string;
 
   while (1)
@@ -1460,6 +1536,9 @@ find_char_unquote (string, stopchars, blank)
 	  while (&p[i] >= string && p[i] == '\\')
 	    --i;
 	  ++i;
+	  /* Only compute the length if really needed.  */
+	  if (string_len == 0)
+	    string_len = strlen (string);
 	  /* The number of backslashes is now -I.
 	     Copy P over itself to swallow half of them.  */
 	  bcopy (&p[i / 2], &p[i], (string_len - (p - string)) - (i / 2) + 1);
@@ -1488,19 +1567,6 @@ find_percent (pattern)
 {
   return find_char_unquote (pattern, "%", 0);
 }
-
-/* Search STRING for an unquoted ; that is not after an unquoted #.  */
-
-static char *
-find_semicolon (string)
-     char *string;
-{
-  char *match = find_char_unquote (string, ";#", 0);
-  if (match != 0 && *match == '#')
-    /* We found a comment before a semicolon.  No match.  */
-    match = 0;
-  return match;
-}
 
 /* Parse a string into a sequence of filenames represented as a
    chain of struct nameseq's in reverse order and return that chain.
@@ -1518,7 +1584,7 @@ find_semicolon (string)
 struct nameseq *
 parse_file_seq (stringp, stopchar, size, strip)
      char **stringp;
-     char stopchar;
+     int stopchar;
      unsigned int size;
      int strip;
 {
@@ -1527,9 +1593,16 @@ parse_file_seq (stringp, stopchar, size, strip)
   register char *p = *stringp;
   char *q;
   char *name;
-  char stopchars[2];
+  char stopchars[3];
+
+#ifdef VMS
+  stopchars[0] = ',';
+  stopchars[1] = stopchar;
+  stopchars[2] = '\0';
+#else
   stopchars[0] = stopchar;
   stopchars[1] = '\0';
+#endif
 
   while (1)
     {
@@ -1539,20 +1612,49 @@ parse_file_seq (stringp, stopchar, size, strip)
 	break;
       if (*p == stopchar)
 	break;
+
       /* Yes, find end of next name.  */
       q = p;
       p = find_char_unquote (q, stopchars, 1);
+#ifdef VMS
+	/* convert comma separated list to space separated */
+      if (p && *p == ',')
+	*p =' ';
+#endif
 #ifdef __MSDOS__
       /* For MS-DOS, skip a "C:\...".  */
       if (stopchar == ':' && p != 0 && p[1] == '\\' && isalpha (p[-1]))
 	p = 0;
+#endif
+#ifdef _AMIGA
+      if (stopchar == ':' && p && *p == ':' &&
+	!(isspace(p[1]) || !p[1] || isspace(p[-1])))
+      {
+	p = find_char_unquote (p+1, stopchars, 1);
+      }
+#endif
+#ifdef WIN32
+      /* For WIN32, skip a "C:\..." or "C:/...". */
+      if (stopchar == ':' &&
+          p != 0 &&
+          (p[1] == '\\' || p[1] == '/') &&
+          isalpha (p[-1])) {
+        p = end_of_token_w32(++p, ':');
+        if (*p == '\0' && p[-1] == ':')
+          p--;
+      }
 #endif
       if (p == 0)
 	p = q + strlen (q);
 
       if (strip)
+#ifdef VMS
+	/* Skip leading `[]'s.  */
+	while (p - q > 2 && q[0] == '[' && q[1] == ']')
+#else
 	/* Skip leading `./'s.  */
 	while (p - q > 2 && q[0] == '.' && q[1] == '/')
+#endif
 	  {
 	    q += 2;		/* Skip "./".  */
 	    while (q < p && *q == '/')
@@ -1563,10 +1665,44 @@ parse_file_seq (stringp, stopchar, size, strip)
       /* Extract the filename just found, and skip it.  */
 
       if (q == p)
-	/* ".///" was stripped to "".  */
+	/* ".///" was stripped to "". */
+#ifdef VMS
+	continue;
+#else
+#ifdef _AMIGA
+	name = savestring ("", 0);
+#else
 	name = savestring ("./", 2);
+#endif
+#endif
       else
+#ifdef VMS
+/* VMS filenames can have a ':' in them but they have to be '\'ed but we need
+ *  to remove this '\' before we can use the filename.
+ * Savestring called because q may be read-only string constant.
+ */
+	{
+	  char *qbase = savestring(q, strlen(q));
+	  char *pbase = qbase + (p-q);
+	  char *q1 = qbase;
+	  char *q2 = q1;
+	  char *p1 = pbase;
+
+	  while (q1 != pbase)
+	    {
+	      if (*q1 == '\\' && *(q1+1) == ':')
+		{
+		  q1++;
+		  p1--;
+		}
+	      *q2++ = *q1++;
+	    }
+	  name = savestring (qbase, p1 - qbase);
+	  free (qbase);
+	}
+#else
 	name = savestring (q, p - q);
+#endif
 
       /* Add it to the front of the chain.  */
       new1 = (struct nameseq *) xmalloc (size);
@@ -1659,7 +1795,7 @@ parse_file_seq (stringp, stopchar, size, strip)
 	    /* Trace back from NEW1 (the end of the list) until N
 	       (the beginning of the list), rewriting each name
 	       with the full archive reference.  */
-	    
+
 	    while (new1 != n)
 	      {
 		name = concat (libname, new1->name, ")");
@@ -1761,7 +1897,7 @@ readline (linebuffer, stream, filename, lineno)
 	  else
 	    break;
 	}
-      
+
       if (!backslash)
 	{
 	  p[-1] = '\0';
@@ -1794,8 +1930,11 @@ construct_include_path (arg_dirs)
      char **arg_dirs;
 {
   register unsigned int i;
+#ifdef VAXC		/* just don't ask ... */
+  stat_t stbuf;
+#else
   struct stat stbuf;
-
+#endif
   /* Table to hold the dirs.  */
 
   register unsigned int defsize = (sizeof (default_include_directories)
@@ -1819,7 +1958,7 @@ construct_include_path (arg_dirs)
 	      dir = expanded;
 	  }
 
-	if (safe_stat (dir, &stbuf) == 0 && S_ISDIR (stbuf.st_mode))
+	if (stat (dir, &stbuf) == 0 && S_ISDIR (stbuf.st_mode))
 	  {
 	    if (idx == max - 1)
 	      {
@@ -1836,7 +1975,7 @@ construct_include_path (arg_dirs)
   /* Now add at the end the standard default dirs.  */
 
   for (i = 0; default_include_directories[i] != 0; ++i)
-    if (safe_stat (default_include_directories[i], &stbuf) == 0
+    if (stat (default_include_directories[i], &stbuf) == 0
 	&& S_ISDIR (stbuf.st_mode))
       dirs[idx++] = default_include_directories[i];
 
@@ -1867,6 +2006,7 @@ char *
 tilde_expand (name)
      char *name;
 {
+#ifndef VMS
   if (name[1] == '/' || name[1] == '\0')
     {
       extern char *getenv ();
@@ -1882,13 +2022,14 @@ tilde_expand (name)
 
 	warn_undefined_variables_flag = save;
       }
-  
+
       is_variable = home_dir[0] != '\0';
       if (!is_variable)
 	{
 	  free (home_dir);
 	  home_dir = getenv ("HOME");
 	}
+#if !defined(_AMIGA) && !defined(WIN32)
       if (home_dir == 0 || home_dir[0] == '\0')
 	{
 	  extern char *getlogin ();
@@ -1901,6 +2042,7 @@ tilde_expand (name)
 		home_dir = p->pw_dir;
 	    }
 	}
+#endif /* !AMIGA && !WIN32 */
       if (home_dir != 0)
 	{
 	  char *new = concat (home_dir, "", name + 1);
@@ -1909,6 +2051,7 @@ tilde_expand (name)
 	  return new;
 	}
     }
+#if !defined(_AMIGA) && !defined(WIN32)
   else
     {
       struct passwd *pwent;
@@ -1926,7 +2069,8 @@ tilde_expand (name)
       else if (userend != 0)
 	*userend = '/';
     }
-
+#endif /* !AMIGA && !WIN32 */
+#endif /* !VMS */
   return 0;
 }
 
@@ -1945,13 +2089,16 @@ multi_glob (chain, size)
      struct nameseq *chain;
      unsigned int size;
 {
+  extern void dir_setup_glob ();
   register struct nameseq *new = 0;
   register struct nameseq *old;
   struct nameseq *nexto;
+  glob_t gl;
+
+  dir_setup_glob (&gl);
 
   for (old = chain; old != 0; old = nexto)
     {
-      glob_t gl;
 #ifndef NO_ARCHIVES
       char *memname;
 #endif
@@ -1983,9 +2130,9 @@ multi_glob (chain, size)
 	}
       else
 	memname = 0;
-#endif
+#endif /* !NO_ARCHIVES */
 
-      switch (glob (old->name, GLOB_NOCHECK, NULL, &gl))
+      switch (glob (old->name, GLOB_NOCHECK|GLOB_ALTDIRFUNC, NULL, &gl))
 	{
 	case 0:			/* Success.  */
 	  {
@@ -2030,7 +2177,7 @@ multi_glob (chain, size)
 		    free (memname);
 		  }
 		else
-#endif
+#endif /* !NO_ARCHIVES */
 		  {
 		    struct nameseq *elt = (struct nameseq *) xmalloc (size);
 		    elt->name = savestring (gl.gl_pathv[i],
@@ -2041,7 +2188,7 @@ multi_glob (chain, size)
 	      }
 	    globfree (&gl);
 	    free (old->name);
-	    free (old);
+	    free ((char *)old);
 	    break;
 	  }
 
