@@ -1,5 +1,5 @@
 /* Basic dependency engine for GNU Make.
-Copyright (C) 1988, 89, 90, 91, 92, 93, 94, 1995 Free Software Foundation, Inc.
+Copyright (C) 1988,89,90,91,92,93,94,95,96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -17,10 +17,10 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
-#include "commands.h"
+#include "filedef.h"
 #include "job.h"
+#include "commands.h"
 #include "dep.h"
-#include "file.h"
 #include <assert.h>
 
 #ifdef HAVE_FCNTL_H
@@ -29,17 +29,33 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <sys/file.h>
 #endif
 
-extern int try_implicit_rule ();
+#ifdef VMS
+#include <starlet.h>
+#endif
+#ifdef WIN32
+#include <io.h>
+#endif
+
+extern int try_implicit_rule PARAMS ((struct file *file, unsigned int depth));
 
 
 /* Incremented when a command is started (under -n, when one would be).  */
 unsigned int commands_started = 0;
 
-static int update_file (), update_file_1 (), check_dep (), touch_file ();
-static void remake_file ();
-static time_t name_mtime ();
-static int library_search ();
-extern time_t f_mtime ();
+static int update_file PARAMS ((struct file *file, unsigned int depth));
+static int update_file_1 PARAMS ((struct file *file, unsigned int depth));
+static int check_dep PARAMS ((struct file *file, unsigned int depth, time_t this_mtime, int *must_make_ptr));
+static int touch_file PARAMS ((struct file *file));
+static void remake_file PARAMS ((struct file *file));
+static time_t name_mtime PARAMS ((char *name));
+static int library_search PARAMS ((char **lib, time_t *mtime_ptr));
+
+extern time_t f_mtime PARAMS ((struct file *file, int search));
+
+#ifdef VMS
+extern int vms_stat PARAMS ((char *name, struct stat *buf));
+#endif
+
 
 /* Remake all the goals in the `struct dep' chain GOALS.  Return -1 if nothing
    was done, 0 if all goals were updated successfully, or 1 if a goal failed.
@@ -100,6 +116,7 @@ update_goal_chain (goals, makefiles)
 	  /* Iterate over all double-colon entries for this file.  */
 	  struct file *file = g->file;
 	  int stop, any_not_updated = 0;
+
 	  for (file = g->file->double_colon ? g->file->double_colon : g->file;
 	       file != NULL;
 	       file = file->prev)
@@ -108,7 +125,6 @@ update_goal_chain (goals, makefiles)
 	      int x;
 	      time_t mtime = MTIME (file);
 	      check_renamed (file);
-
 	      if (makefiles)
 		{
 		  if (file->cmd_target)
@@ -133,7 +149,7 @@ update_goal_chain (goals, makefiles)
 		 by calling update_file above.  We check this flag below to
 		 decide when to give an "up to date" diagnostic.  */
 	      g->changed += commands_started - ocommands_started;
-	    
+
 	      stop = 0;
 	      if (x != 0 || file->updated)
 		{
@@ -195,14 +211,10 @@ update_goal_chain (goals, makefiles)
 		  && file->update_status == 0 && !g->changed
 		  /* Never give a message under -s or -q.  */
 		  && !silent_flag && !question_flag)
-		{
-		  if (file->phony || file->cmds == 0)
-		    message ("Nothing to be done for `%s'.",
-			     file->name);
-		  else
-		    message ("`%s' is up to date.", file->name);
-		  fflush (stdout);
-		}
+		message (1, ((file->phony || file->cmds == 0)
+			     ? "Nothing to be done for `%s'."
+			     : "`%s' is up to date."),
+			 file->name);
 
 	      /* This goal is finished.  Remove it from the chain.  */
 	      if (lastgoal == 0)
@@ -233,7 +245,6 @@ update_goal_chain (goals, makefiles)
       just_print_flag = n;
       job_slots = j;
     }
-
   return status;
 }
 
@@ -593,7 +604,9 @@ update_file_1 (file, depth)
 
 /* Set FILE's `updated' flag and re-check its mtime and the mtime's of all
    files listed in its `also_make' member.  Under -t, this function also
-   touches FILE.  */
+   touches FILE.
+
+   On return, FILE->update_status will no longer be -1 if it was.  */
 
 void
 notice_finished_file (file)
@@ -668,6 +681,10 @@ notice_finished_file (file)
 	     never be done because the target is already updated.  */
 	  (void) f_mtime (d->file, 0);
       }
+  else if (file->update_status == -1)
+    /* Nothing was done for FILE, but it needed nothing done.
+       So mark it now as "succeeded".  */
+    file->update_status = 0;
 }
 
 /* Check whether another file (whose mtime is THIS_MTIME)
@@ -704,49 +721,78 @@ check_dep (file, depth, this_mtime, must_make_ptr)
     }
   else
     {
-      /* FILE is an intermediate file.
-	 Update all non-intermediate files we depend on, if necessary,
-	 and see whether any of them is more recent than the file
-	 on whose behalf we are checking.  */
-      register struct dep *lastd;
-      lastd = 0;
-      d = file->deps;
-      while (d != 0)
+      /* FILE is an intermediate file.  */
+      time_t mtime;
+
+      if (!file->phony && file->cmds == 0 && !file->tried_implicit
+	  && file->secondary)
 	{
-	  if (d->file->updating)
+	  if (try_implicit_rule (file, depth))
+	    DEBUGPR ("Found an implicit rule for `%s'.\n");
+	  else
+	    DEBUGPR ("No implicit rule found for `%s'.\n");
+	  file->tried_implicit = 1;
+	}
+      if (file->cmds == 0 && !file->is_target && file->secondary
+	  && default_file != 0 && default_file->cmds != 0)
+	{
+	  DEBUGPR ("Using default commands for `%s'.\n");
+	  file->cmds = default_file->cmds;
+	}
+
+      /* If the intermediate file actually exists
+	 and is newer, then we should remake from it.  */
+      check_renamed (file);
+      mtime = file_mtime (file);
+      check_renamed (file);
+      if (mtime > this_mtime)
+	*must_make_ptr = 1;
+	  /* Otherwise, update all non-intermediate files we depend on,
+	     if necessary, and see whether any of them is more
+	     recent than the file on whose behalf we are checking.  */
+      else
+	{
+	  register struct dep *lastd;
+
+	  lastd = 0;
+	  d = file->deps;
+	  while (d != 0)
 	    {
-	      error ("Circular %s <- %s dependency dropped.",
-		     file->name, d->file->name);
-	      if (lastd == 0)
+	      if (d->file->updating)
 		{
-		  file->deps = d->next;
-		  free ((char *) d);
-		  d = file->deps;
+		  error ("Circular %s <- %s dependency dropped.",
+			 file->name, d->file->name);
+		  if (lastd == 0)
+		    {
+		      file->deps = d->next;
+		      free ((char *) d);
+		      d = file->deps;
+		    }
+		  else
+		    {
+		      lastd->next = d->next;
+		      free ((char *) d);
+		      d = lastd->next;
+		    }
+		  continue;
 		}
-	      else
-		{
-		  lastd->next = d->next;
-		  free ((char *) d);
-		  d = lastd->next;
-		}
-	      continue;
+
+	      d->file->parent = file;
+	      dep_status |= check_dep (d->file, depth, this_mtime, must_make_ptr);
+	      check_renamed (d->file);
+	      if (dep_status != 0 && !keep_going_flag)
+		break;
+
+	      if (d->file->command_state == cs_running
+		  || d->file->command_state == cs_deps_running)
+		/* Record that some of FILE's dependencies are still being made.
+		   This tells the upper levels to wait on processing it until
+		   the commands are finished.  */
+		set_command_state (file, cs_deps_running);
+
+	      lastd = d;
+	      d = d->next;
 	    }
-
-	  d->file->parent = file;
-	  dep_status |= check_dep (d->file, depth, this_mtime, must_make_ptr);
-	  check_renamed (d->file);
-	  if (dep_status != 0 && !keep_going_flag)
-	    break;
-
-	  if (d->file->command_state == cs_running
-	      || d->file->command_state == cs_deps_running)
-	    /* Record that some of FILE's dependencies are still being made.
-	       This tells the upper levels to wait on processing it until
-	       the commands are finished.  */
-	    set_command_state (file, cs_deps_running);
-
-	  lastd = d;
-	  d = d->next;
 	}
     }
 
@@ -763,10 +809,7 @@ touch_file (file)
      register struct file *file;
 {
   if (!silent_flag)
-    {
-      printf ("touch %s\n", file->name);
-      fflush (stdout);
-    }
+    message (0, "touch %s", file->name);
 
 #ifndef	NO_ARCHIVES
   if (ar_name (file->name))
@@ -980,6 +1023,35 @@ f_mtime (file, search)
 	}
     }
 
+  {
+    /* Files can have bogus timestamps that nothing newly made will be
+       "newer" than.  Updating their dependents could just result in loops.
+       So notify the user of the anomaly by treating future files as
+       unrecoverably absent.  */
+
+    static time_t now;
+    if (mtime != -1 && mtime > now && ! file->updated)
+      {
+	/* This file's time appears to be in the future.
+	   Update our concept of the present, and compare again.  */
+#ifdef VMS
+	/* Handle vms 64bit to 32bit time hack introduced in vms_stat ... */
+	static unsigned long vms_now[2]; /* assumes 32 bit long ! */
+	sys$gettim (vms_now);
+	now = ((vms_now[0]>>24) & 0xff) + ((vms_now[1]<<8) & 0xffffff00);
+	if (mtime > now)
+#else
+	extern time_t time ();
+	if (mtime > time (&now))
+#endif
+	  {
+	    error ("*** File `%s' has modification time in the future",
+		   file->name);
+	    file->update_status = 2;
+	  }
+      }
+  }
+
   /* Store the mtime into all the entries for this file.  */
   if (file->double_colon)
     file = file->double_colon;
@@ -1001,7 +1073,11 @@ name_mtime (name)
 {
   struct stat st;
 
-  if (safe_stat (name, &st) < 0)
+#ifdef VMS
+  if (vms_stat (name, &st) < 0)
+#else
+  if (stat (name, &st) < 0)
+#endif
     return (time_t) -1;
 
   return (time_t) st.st_mtime;
@@ -1019,8 +1095,17 @@ library_search (lib, mtime_ptr)
 {
   static char *dirs[] =
     {
+#ifndef _AMIGA
       "/lib",
       "/usr/lib",
+#endif
+#if defined(WIN32) && !defined(LIBDIR)
+/*
+ * This is completely up to the user at product install time. Just define
+ * a placeholder.
+ */
+#define LIBDIR "."
+#endif
       LIBDIR,			/* Defined by configuration.  */
       0
     };
@@ -1034,7 +1119,11 @@ library_search (lib, mtime_ptr)
 
   /* Look first for `libNAME.a' in the current directory.  */
 
+#ifndef _AMIGA
   sprintf (buf, "lib%s.a", libname);
+#else
+  sprintf (buf, "%s.lib", libname);
+#endif
   mtime = name_mtime (buf);
   if (mtime != (time_t) -1)
     {
@@ -1058,7 +1147,11 @@ library_search (lib, mtime_ptr)
 
   for (dp = dirs; *dp != 0; ++dp)
     {
+#ifndef _AMIGA
       sprintf (buf, "%s/lib%s.a", *dp, libname);
+#else
+      sprintf (buf, "%s/%s.lib", *dp, libname);
+#endif
       mtime = name_mtime (buf);
       if (mtime != (time_t) -1)
 	{
