@@ -1,5 +1,5 @@
 /* Internals of variables for GNU Make.
-Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
+Copyright (C) 1988, 89, 90, 91, 92, 93, 94, 96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -17,10 +17,14 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
+#include "dep.h"
+#include "filedef.h"
+#include "job.h"
 #include "commands.h"
 #include "variable.h"
-#include "dep.h"
-#include "file.h"
+#ifdef WIN32
+#include "pathstuff.h"
+#endif
 
 /* Hash table of all global variable definitions.  */
 
@@ -39,6 +43,11 @@ static struct variable_set global_variable_set
 static struct variable_set_list global_setlist
   = { 0, &global_variable_set };
 struct variable_set_list *current_variable_set_list = &global_setlist;
+
+static struct variable *define_variable_in_set PARAMS ((char *name, unsigned int length,
+							char *value, enum variable_origin origin,
+							int recursive, struct variable_set *set));
+
 
 /* Implement variables.  */
 
@@ -345,13 +354,26 @@ merge_variable_set_lists (setlist0, setlist1)
 void
 define_automatic_variables ()
 {
+#ifdef WIN32
+  extern char* default_shell;
+#else
   extern char default_shell[];
+#endif
   register struct variable *v;
-  char buf[100];
+  char buf[200];
 
   sprintf (buf, "%u", makelevel);
   (void) define_variable ("MAKELEVEL", 9, buf, o_env, 0);
 
+  sprintf (buf, "%s%s%s",
+	   version_string,
+	   (remote_description == 0 || remote_description[0] == '\0')
+	   ? "" : "-",
+	   (remote_description == 0 || remote_description[0] == '\0')
+	   ? "" : remote_description);
+  (void) define_variable ("MAKE_VERSION", 12, buf, o_default, 0);
+
+
   /* This won't override any definition, but it
      will provide one if there isn't one there.  */
   v = define_variable ("SHELL", 5, default_shell, o_default, 0);
@@ -372,18 +394,20 @@ define_automatic_variables ()
   /* Define the magic D and F variables in terms of
      the automatic variables they are variations of.  */
 
-  define_variable ("@D", 2, "$(dir $@)", o_automatic, 1);
-  define_variable ("%D", 2, "$(dir $%)", o_automatic, 1);
-  define_variable ("*D", 2, "$(dir $*)", o_automatic, 1);
-  define_variable ("<D", 2, "$(dir $<)", o_automatic, 1);
-  define_variable ("?D", 2, "$(dir $?)", o_automatic, 1);
-  define_variable ("^D", 2, "$(dir $^)", o_automatic, 1);
+  define_variable ("@D", 2, "$(patsubst %/,%,$(dir $@))", o_automatic, 1);
+  define_variable ("%D", 2, "$(patsubst %/,%,$(dir $%))", o_automatic, 1);
+  define_variable ("*D", 2, "$(patsubst %/,%,$(dir $*))", o_automatic, 1);
+  define_variable ("<D", 2, "$(patsubst %/,%,$(dir $<))", o_automatic, 1);
+  define_variable ("?D", 2, "$(patsubst %/,%,$(dir $?))", o_automatic, 1);
+  define_variable ("^D", 2, "$(patsubst %/,%,$(dir $^))", o_automatic, 1);
+  define_variable ("+D", 2, "$(patsubst %/,%,$(dir $+))", o_automatic, 1);
   define_variable ("@F", 2, "$(notdir $@)", o_automatic, 1);
   define_variable ("%F", 2, "$(notdir $%)", o_automatic, 1);
   define_variable ("*F", 2, "$(notdir $*)", o_automatic, 1);
   define_variable ("<F", 2, "$(notdir $<)", o_automatic, 1);
   define_variable ("?F", 2, "$(notdir $?)", o_automatic, 1);
   define_variable ("^F", 2, "$(notdir $^)", o_automatic, 1);
+  define_variable ("+F", 2, "$(notdir $+)", o_automatic, 1);
 }
 
 int export_all_variables;
@@ -446,7 +470,6 @@ target_environment (file)
 	  register struct variable *v;
 	  for (v = set->table[i]; v != 0; v = v->next)
 	    {
-	      extern char *getenv ();
 	      unsigned int j = i % buckets;
 	      register struct variable_bucket *ov;
 	      register char *p = v->name;
@@ -464,10 +487,9 @@ target_environment (file)
 		    /* Only export default variables by explicit request.  */
 		    continue;
 
-		  if (!export_all_variables
+		  if (! export_all_variables
 		      && v->origin != o_command
-		      && v->origin != o_env && v->origin != o_env_override
-		      && !(v->origin == o_file && getenv (p) != 0))
+		      && v->origin != o_env && v->origin != o_env_override)
 		    continue;
 
 		  if (*p != '_' && (*p < 'A' || *p > 'Z')
@@ -524,11 +546,25 @@ target_environment (file)
 	      && v->origin != o_env && v->origin != o_env_override)
 	    {
 	      char *value = recursively_expand (v);
+#ifdef WIN32
+              if (strcmp(v->name, "Path") == 0 ||
+                  strcmp(v->name, "PATH") == 0)
+                convert_Path_to_win32(value, ';');
+#endif
 	      result[nvariables++] = concat (v->name, "=", value);
 	      free (value);
 	    }
 	  else
+#ifdef WIN32
+          {
+            if (strcmp(v->name, "Path") == 0 ||
+                strcmp(v->name, "PATH") == 0)
+              convert_Path_to_win32(v->value, ';');
+            result[nvariables++] = concat (v->name, "=", v->value);
+          }
+#else
 	    result[nvariables++] = concat (v->name, "=", v->value);
+#endif
 	}
     }
   result[nvariables] = (char *) xmalloc (100);
@@ -596,11 +632,39 @@ try_variable_definition (filename, lineno, line, origin)
 	  flavor = append;
 	  break;
 	}
+      else if (c == '$')
+	{
+	  /* This might begin a variable expansion reference.  Make sure we
+	     don't misrecognize chars inside the reference as =, := or +=.  */
+	  char closeparen;
+	  int count;
+	  c = *p++;
+	  if (c == '(')
+	    closeparen = ')';
+	  else if (c == '{')
+	    closeparen = '}';
+	  else
+	    continue;		/* Nope.  */
+
+	  /* P now points past the opening paren or brace.
+	     Count parens or braces until it is matched.  */
+	  count = 0;
+	  for (; *p != '\0'; ++p)
+	    {
+	      if (*p == c)
+		++count;
+	      else if (*p == closeparen && --count < 0)
+		{
+		  ++p;
+		  break;
+		}
+	    }
+	}
     }
 
-  while (isblank (end[-1]))
-    --end;
   beg = next_token (line);
+  while (end > beg && isblank (end[-1]))
+    --end;
   p = next_token (p);
 
   /* Expand the name, so "$(foo)bar = baz" works.  */
@@ -624,7 +688,7 @@ try_variable_definition (filename, lineno, line, origin)
     case bogus:
       /* Should not be possible.  */
       abort ();
-      break;
+      return 0;
     case simple:
       /* A simple variable definition "var := value".  Expand the value.  */
       value = variable_expand (p);
@@ -786,6 +850,14 @@ print_variable_set (set, prefix)
 max %u in one bucket.\n",
 	      (double) nvariables / (double) set->buckets,
 	      per_bucket);
+#else
+      {
+	int f = (nvariables * 1000 + 5) / set->buckets;
+	printf ("# average of %d.%d variables per bucket, \
+max %u in one bucket.\n",
+	      f/10, f%10,
+	      per_bucket);
+      }
 #endif
     }
 }
@@ -811,3 +883,30 @@ print_file_variables (file)
   if (file->variables != 0)
     print_variable_set (file->variables->set, "# ");
 }
+
+#ifdef WIN32
+void
+sync_Path_environment(void)
+{
+    char* path = allocated_variable_expand("$(Path)");
+    static char* environ_path = NULL;
+
+    if (!path)
+        return;
+
+    /*
+     * If done this before, don't leak memory unnecessarily.
+     * Free the previous entry before allocating new one.
+     */
+    if (environ_path)
+        free(environ_path);
+
+    /*
+     * Create something WIN32 world can grok
+     */
+    convert_Path_to_win32(path, ';');
+    environ_path = concat("Path", "=", path);
+    putenv(environ_path);
+    free(path);
+}
+#endif
