@@ -1,5 +1,5 @@
 /* Implementation of pattern-matching file search paths for GNU Make.
-Copyright (C) 1988, 1989, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
+Copyright (C) 1988, 89, 91, 92, 93, 94, 95, 96 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -17,8 +17,11 @@ along with GNU Make; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "make.h"
-#include "file.h"
+#include "filedef.h"
 #include "variable.h"
+#ifdef WIN32
+#include "pathstuff.h"
+#endif
 
 
 /* Structure used to represent a selective VPATH searchpath.  */
@@ -41,7 +44,7 @@ static struct vpath *vpaths;
 
 static struct vpath *general_vpath;
 
-static int selective_vpath_search ();
+static int selective_vpath_search PARAMS ((struct vpath *path, char **file, time_t *mtime_ptr));
 
 /* Reverse the chain of selective VPATH lists so they
    will be searched in the order given in the makefiles
@@ -170,6 +173,10 @@ construct_vpath_list (pattern, dirpath)
       return;
     }
 
+#ifdef WIN32
+    convert_vpath_to_win32(dirpath, ';');
+#endif
+
   /* Figure out the maximum number of VPATH entries and
      put it in MAXELEM.  We start with 2, one before the
      first colon and one nil, the list terminator and
@@ -279,7 +286,12 @@ vpath_search (file, mtime_ptr)
   /* If there are no VPATH entries or FILENAME starts at the root,
      there is nothing we can do.  */
 
-  if (**file == '/' || (vpaths == 0 && general_vpath == 0))
+  if (**file == '/'
+#ifdef WIN32
+      || **file == '\\' 
+      || (*file)[1] == ':'
+#endif
+      || (vpaths == 0 && general_vpath == 0))
     return 0;
 
   for (v = vpaths; v != 0; v = v->next)
@@ -331,6 +343,10 @@ selective_vpath_search (path, file, mtime_ptr)
      pointer to the name-within-directory and FLEN is its length.  */
 
   n = rindex (*file, '/');
+#ifdef WIN32
+  if (!n)
+    n = rindex(*file,, '\\');
+#endif
   name_dplen = n != 0 ? n - *file : 0;
   filename = name_dplen > 0 ? n + 1 : *file;
   if (name_dplen > 0)
@@ -340,7 +356,7 @@ selective_vpath_search (path, file, mtime_ptr)
      a slash, the directory prefix that came with *FILE,
      another slash (although this one may not always be
      necessary), the filename, and a null terminator.  */
-  name = (char *) alloca (maxvpath + 1 + name_dplen + 1 + flen + 1);
+  name = (char *) xmalloc (maxvpath + 1 + name_dplen + 1 + flen + 1);
 
   /* Try each VPATH entry.  */
   for (i = 0; vpath[i] != 0; ++i)
@@ -419,7 +435,7 @@ selective_vpath_search (path, file, mtime_ptr)
 	  *n = '/';
 
 	  if (!exists_in_cache	/* Makefile-mentioned file need not exist.  */
-	      || safe_stat (name, &st) == 0) /* Does it really exist?  */
+	      || stat (name, &st) == 0) /* Does it really exist?  */
 	    {
 	      /* We have found a file.
 		 Store the name we found into *FILE for the caller.  */
@@ -432,6 +448,7 @@ selective_vpath_search (path, file, mtime_ptr)
 		   we record a zero modtime to indicate this.  */
 		*mtime_ptr = exists_in_cache ? st.st_mtime : (time_t) 0;
 
+	      free (name);
 	      return 1;
 	    }
 	  else
@@ -439,6 +456,7 @@ selective_vpath_search (path, file, mtime_ptr)
 	}
     }
 
+  free (name);
   return 0;
 }
 
